@@ -22,9 +22,8 @@ from chia.util.config import (
     save_config,
     unflatten_properties,
 )
-from chia.util.ints import uint32
 from chia.util.keychain import Keychain
-from chia.util.path import mkdir
+from chia.util.path import mkdir, path_from_root
 from chia.util.ssl_check import (
     DEFAULT_PERMISSIONS_CERT_FILE,
     DEFAULT_PERMISSIONS_KEY_FILE,
@@ -33,10 +32,16 @@ from chia.util.ssl_check import (
     check_and_fix_permissions_for_ssl_file,
     fix_ssl,
 )
-from chia.wallet.derive_keys import master_sk_to_pool_sk, master_sk_to_wallet_sk
+from chia.wallet.derive_keys import (
+    master_sk_to_pool_sk,
+    master_sk_to_wallet_sk_intermediate,
+    master_sk_to_wallet_sk_unhardened_intermediate,
+    _derive_path,
+    _derive_path_unhardened,
+)
 from chia.cmds.configure import configure
 
-private_node_names = {"full_node", "wallet", "farmer", "harvester", "timelord", "daemon"}
+private_node_names = {"full_node", "wallet", "farmer", "harvester", "timelord", "crawler", "daemon"}
 public_node_names = {"full_node", "wallet", "farmer", "introducer", "timelord"}
 
 
@@ -74,19 +79,39 @@ def check_keys(new_root: Path, keychain: Optional[Keychain] = None) -> None:
     all_targets = []
     stop_searching_for_farmer = "xch_target_address" not in config["farmer"]
     stop_searching_for_pool = "xch_target_address" not in config["pool"]
-    number_of_ph_to_search = 500
+    number_of_ph_to_search = 50
     selected = config["selected_network"]
     prefix = config["network_overrides"]["config"][selected]["address_prefix"]
+
+    intermediates = {}
+    for sk, _ in all_sks:
+        intermediates[bytes(sk)] = {
+            "observer": master_sk_to_wallet_sk_unhardened_intermediate(sk),
+            "non-observer": master_sk_to_wallet_sk_intermediate(sk),
+        }
+
     for i in range(number_of_ph_to_search):
         if stop_searching_for_farmer and stop_searching_for_pool and i > 0:
             break
         for sk, _ in all_sks:
+            intermediate_n = intermediates[bytes(sk)]["non-observer"]
+            intermediate_o = intermediates[bytes(sk)]["observer"]
+
             all_targets.append(
-                encode_puzzle_hash(create_puzzlehash_for_pk(master_sk_to_wallet_sk(sk, uint32(i)).get_g1()), prefix)
+                encode_puzzle_hash(
+                    create_puzzlehash_for_pk(_derive_path_unhardened(intermediate_o, [i]).get_g1()), prefix
+                )
             )
-            if all_targets[-1] == config["farmer"].get("xch_target_address"):
+            all_targets.append(
+                encode_puzzle_hash(create_puzzlehash_for_pk(_derive_path(intermediate_n, [i]).get_g1()), prefix)
+            )
+            if all_targets[-1] == config["farmer"].get("xch_target_address") or all_targets[-2] == config["farmer"].get(
+                "xch_target_address"
+            ):
                 stop_searching_for_farmer = True
-            if all_targets[-1] == config["pool"].get("xch_target_address"):
+            if all_targets[-1] == config["pool"].get("xch_target_address") or all_targets[-2] == config["pool"].get(
+                "xch_target_address"
+            ):
                 stop_searching_for_pool = True
 
     # Set the destinations, if necessary
@@ -99,7 +124,7 @@ def check_keys(new_root: Path, keychain: Optional[Keychain] = None) -> None:
         updated_target = True
     elif config["farmer"]["xch_target_address"] not in all_targets:
         print(
-            f"WARNING: using a farmer address which we don't have the private"
+            f"WARNING: using a farmer address which we might not have the private"
             f" keys for. We searched the first {number_of_ph_to_search} addresses. Consider overriding "
             f"{config['farmer']['xch_target_address']} with {all_targets[0]}"
         )
@@ -112,7 +137,7 @@ def check_keys(new_root: Path, keychain: Optional[Keychain] = None) -> None:
         updated_target = True
     elif config["pool"]["xch_target_address"] not in all_targets:
         print(
-            f"WARNING: using a pool address which we don't have the private"
+            f"WARNING: using a pool address which we might not have the private"
             f" keys for. We searched the first {number_of_ph_to_search} addresses. Consider overriding "
             f"{config['pool']['xch_target_address']} with {all_targets[0]}"
         )
@@ -255,7 +280,13 @@ def copy_cert_files(cert_path: Path, new_path: Path):
         check_and_fix_permissions_for_ssl_file(new_path_child, RESTRICT_MASK_KEY_FILE, DEFAULT_PERMISSIONS_KEY_FILE)
 
 
-def init(create_certs: Optional[Path], root_path: Path, fix_ssl_permissions: bool = False, testnet: bool = False):
+def init(
+    create_certs: Optional[Path],
+    root_path: Path,
+    fix_ssl_permissions: bool = False,
+    testnet: bool = False,
+    v1_db: bool = False,
+):
     if create_certs is not None:
         if root_path.exists():
             if os.path.isdir(create_certs):
@@ -272,7 +303,13 @@ def init(create_certs: Optional[Path], root_path: Path, fix_ssl_permissions: boo
             print(f"** {root_path} does not exist. Executing core init **")
             # sanity check here to prevent infinite recursion
             if (
-                chia_init(root_path, fix_ssl_permissions=fix_ssl_permissions, testnet=testnet) == 0
+                chia_init(
+                    root_path,
+                    fix_ssl_permissions=fix_ssl_permissions,
+                    testnet=testnet,
+                    v1_db=v1_db,
+                )
+                == 0
                 and root_path.exists()
             ):
                 return init(create_certs, root_path, fix_ssl_permissions)
@@ -280,7 +317,7 @@ def init(create_certs: Optional[Path], root_path: Path, fix_ssl_permissions: boo
             print(f"** {root_path} was not created. Exiting **")
             return -1
     else:
-        return chia_init(root_path, fix_ssl_permissions=fix_ssl_permissions, testnet=testnet)
+        return chia_init(root_path, fix_ssl_permissions=fix_ssl_permissions, testnet=testnet, v1_db=v1_db)
 
 
 def chia_version_number() -> Tuple[str, str, str, str]:
@@ -343,7 +380,12 @@ def chia_full_version_str() -> str:
 
 
 def chia_init(
-    root_path: Path, *, should_check_keys: bool = True, fix_ssl_permissions: bool = False, testnet: bool = False
+    root_path: Path,
+    *,
+    should_check_keys: bool = True,
+    fix_ssl_permissions: bool = False,
+    testnet: bool = False,
+    v1_db: bool = False,
 ):
     """
     Standard first run initialization or migration steps. Handles config creation,
@@ -365,7 +407,23 @@ def chia_init(
         # This is reached if CHIA_ROOT is set, or if user has run chia init twice
         # before a new update.
         if testnet:
-            configure(root_path, "", "", "", "", "", "", "", "", testnet="true", peer_connect_timeout="")
+            configure(
+                root_path,
+                set_farmer_peer="",
+                set_node_introducer="",
+                set_fullnode_port="",
+                set_harvester_port="",
+                set_log_level="",
+                enable_upnp="",
+                set_outbound_peer_count="",
+                set_peer_count="",
+                testnet="true",
+                peer_connect_timeout="",
+                crawler_db_path="",
+                crawler_minimum_version_count=None,
+                seeder_domain_name="",
+                seeder_nameserver="",
+            )
         if fix_ssl_permissions:
             fix_ssl(root_path)
         if should_check_keys:
@@ -375,12 +433,48 @@ def chia_init(
 
     create_default_chia_config(root_path)
     if testnet:
-        configure(root_path, "", "", "", "", "", "", "", "", testnet="true", peer_connect_timeout="")
+        configure(
+            root_path,
+            set_farmer_peer="",
+            set_node_introducer="",
+            set_fullnode_port="",
+            set_harvester_port="",
+            set_log_level="",
+            enable_upnp="",
+            set_outbound_peer_count="",
+            set_peer_count="",
+            testnet="true",
+            peer_connect_timeout="",
+            crawler_db_path="",
+            crawler_minimum_version_count=None,
+            seeder_domain_name="",
+            seeder_nameserver="",
+        )
     create_all_ssl(root_path)
     if fix_ssl_permissions:
         fix_ssl(root_path)
     if should_check_keys:
         check_keys(root_path)
+
+    config: Dict
+    if v1_db:
+        config = load_config(root_path, "config.yaml")
+        db_pattern = config["full_node"]["database_path"]
+        new_db_path = db_pattern.replace("_v2_", "_v1_")
+        config["full_node"]["database_path"] = new_db_path
+        save_config(root_path, "config.yaml", config)
+    else:
+        config = load_config(root_path, "config.yaml")["full_node"]
+        db_path_replaced: str = config["database_path"].replace("CHALLENGE", config["selected_network"])
+        db_path = path_from_root(root_path, db_path_replaced)
+        mkdir(db_path.parent)
+        import sqlite3
+
+        with sqlite3.connect(db_path) as connection:
+            connection.execute("CREATE TABLE database_version(version int)")
+            connection.execute("INSERT INTO database_version VALUES (2)")
+            connection.commit()
+
     print("")
     print("To see your keys, run 'chia keys show --show-mnemonic-seed'")
 
