@@ -1,17 +1,17 @@
 import logging
 import time
 from typing import Dict, List, Optional
-from clvm_rs import STRICT_MODE
+from clvm_rs import MEMPOOL_MODE, COND_CANON_INTS, NO_NEG_DIV
 
+from clvm.casts import int_from_bytes, int_to_bytes
 from chia.consensus.cost_calculator import NPCResult
+from chia.consensus.default_constants import DEFAULT_CONSTANTS
 from chia.full_node.generator import create_generator_args, setup_generator_args
-from chia.types.blockchain_format.program import NIL
 from chia.types.coin_record import CoinRecord
+from chia.types.condition_opcodes import ConditionOpcode
 from chia.types.condition_with_args import ConditionWithArgs
 from chia.types.generator_types import BlockGenerator
 from chia.types.name_puzzle_condition import NPC
-from chia.util.clvm import int_from_bytes
-from chia.util.condition_tools import ConditionOpcode
 from chia.util.errors import Err
 from chia.util.ints import uint32, uint64, uint16
 from chia.wallet.puzzles.generator_loader import GENERATOR_FOR_SINGLE_COIN_MOD
@@ -19,7 +19,6 @@ from chia.wallet.puzzles.rom_bootstrap_generator import get_generator
 
 GENERATOR_MOD = get_generator()
 
-
 log = logging.getLogger(__name__)
 
 
@@ -88,30 +87,97 @@ def mempool_assert_relative_time_exceeds(
     return None
 
 
+def add_int_cond(
+    conds: Dict[ConditionOpcode, List[ConditionWithArgs]],
+    op: ConditionOpcode,
+    arg: int,
+):
+    if op not in conds:
+        conds[op] = []
+    conds[op].append(ConditionWithArgs(op, [int_to_bytes(arg)]))
+
+
+def add_cond(
+    conds: Dict[ConditionOpcode, List[ConditionWithArgs]],
+    op: ConditionOpcode,
+    args: List[bytes],
+):
+    if op not in conds:
+        conds[op] = []
+    conds[op].append(ConditionWithArgs(op, args))
+
+
+def unwrap(x: Optional[uint32]) -> uint32:
+    assert x is not None
+    return x
+
+
 def get_name_puzzle_conditions(
-    generator: BlockGenerator, max_cost: int, *, cost_per_byte: int, safe_mode: bool
+    generator: BlockGenerator, max_cost: int, *, cost_per_byte: int, mempool_mode: bool, height: Optional[uint32] = None
 ) -> NPCResult:
     block_program, block_program_args = setup_generator_args(generator)
-    max_cost -= len(bytes(generator.program)) * cost_per_byte
+    size_cost = len(bytes(generator.program)) * cost_per_byte
+    max_cost -= size_cost
     if max_cost < 0:
         return NPCResult(uint16(Err.INVALID_BLOCK_COST.value), [], uint64(0))
 
-    flags = STRICT_MODE if safe_mode else 0
+    # in mempool mode, the height doesn't matter, because it's always strict.
+    # But otherwise, height must be specified to know which rules to apply
+    assert mempool_mode or height is not None
+
+    # mempool mode also has these rules apply
+    assert (MEMPOOL_MODE & COND_CANON_INTS) != 0
+    assert (MEMPOOL_MODE & NO_NEG_DIV) != 0
+
+    if mempool_mode:
+        flags = MEMPOOL_MODE
+    elif unwrap(height) >= DEFAULT_CONSTANTS.SOFT_FORK_HEIGHT:
+        # conditions must use integers in canonical encoding (i.e. no redundant
+        # leading zeros)
+        # the division operator may not be used with negative operands
+        flags = COND_CANON_INTS | NO_NEG_DIV
+    else:
+        flags = 0
+
     try:
-        err, result, clvm_cost = GENERATOR_MOD.run_as_generator(max_cost, flags, block_program, block_program_args)
+        err, result = GENERATOR_MOD.run_as_generator(max_cost, flags, block_program, block_program_args)
+
         if err is not None:
+            assert err != 0
             return NPCResult(uint16(err), [], uint64(0))
-        else:
-            npc_list = []
-            for r in result:
-                conditions = []
-                for c in r.conditions:
-                    cwa = []
-                    for cond_list in c[1]:
-                        cwa.append(ConditionWithArgs(ConditionOpcode(bytes([cond_list.opcode])), cond_list.vars))
-                    conditions.append((ConditionOpcode(bytes([c[0]])), cwa))
-                npc_list.append(NPC(r.coin_name, r.puzzle_hash, conditions))
-            return NPCResult(None, npc_list, uint64(clvm_cost))
+
+        first = True
+        npc_list = []
+        for r in result.spends:
+            conditions: Dict[ConditionOpcode, List[ConditionWithArgs]] = {}
+            if r.height_relative is not None:
+                add_int_cond(conditions, ConditionOpcode.ASSERT_HEIGHT_RELATIVE, r.height_relative)
+            if r.seconds_relative > 0:
+                add_int_cond(conditions, ConditionOpcode.ASSERT_SECONDS_RELATIVE, r.seconds_relative)
+            for cc in r.create_coin:
+                if cc[2] == b"":
+                    add_cond(conditions, ConditionOpcode.CREATE_COIN, [cc[0], int_to_bytes(cc[1])])
+                else:
+                    add_cond(conditions, ConditionOpcode.CREATE_COIN, [cc[0], int_to_bytes(cc[1]), cc[2]])
+            for sig in r.agg_sig_me:
+                add_cond(conditions, ConditionOpcode.AGG_SIG_ME, [sig[0], sig[1]])
+
+            # all conditions that aren't tied to a specific spent coin, we roll into the first one
+            if first:
+                first = False
+                if result.reserve_fee > 0:
+                    add_int_cond(conditions, ConditionOpcode.RESERVE_FEE, result.reserve_fee)
+                if result.height_absolute > 0:
+                    add_int_cond(conditions, ConditionOpcode.ASSERT_HEIGHT_ABSOLUTE, result.height_absolute)
+                if result.seconds_absolute > 0:
+                    add_int_cond(conditions, ConditionOpcode.ASSERT_SECONDS_ABSOLUTE, result.seconds_absolute)
+                for sig in result.agg_sig_unsafe:
+                    add_cond(conditions, ConditionOpcode.AGG_SIG_UNSAFE, [sig[0], sig[1]])
+
+            npc_list.append(NPC(r.coin_id, r.puzzle_hash, [(op, cond) for op, cond in conditions.items()]))
+
+        return NPCResult(None, npc_list, uint64(result.cost + size_cost))
+
     except BaseException as e:
         log.debug(f"get_name_puzzle_condition failed: {e}")
         return NPCResult(uint16(Err.GENERATOR_RUNTIME_ERROR.value), [], uint64(0))
@@ -120,10 +186,7 @@ def get_name_puzzle_conditions(
 def get_puzzle_and_solution_for_coin(generator: BlockGenerator, coin_name: bytes, max_cost: int):
     try:
         block_program = generator.program
-        if not generator.generator_args:
-            block_program_args = [NIL]
-        else:
-            block_program_args = create_generator_args(generator.generator_refs())
+        block_program_args = create_generator_args(generator.generator_refs)
 
         cost, result = GENERATOR_FOR_SINGLE_COIN_MOD.run_with_cost(
             max_cost, block_program, block_program_args, coin_name
