@@ -59,7 +59,7 @@ def main() -> None:
             sys.argv.remove("-D")  # Remove -D to avoid conflicting with load_config_cli's argparse usage
         config = load_config_cli(DEFAULT_ROOT_PATH, "config.yaml", SERVICE_NAME)
         config["database_path"] = config["simulator_database_path"]
-        config["peer_db_path"] = config["simulator_peer_db_path"]
+        config["peers_file_path"] = config["simulator_peers_file_path"]
         config["introducer_peer"]["host"] = "127.0.0.1"
         config["introducer_peer"]["port"] = 58555
         config["selected_network"] = "testnet0"
