@@ -1,4 +1,5 @@
 import argparse
+import logging
 import os
 import shutil
 import sys
@@ -7,9 +8,13 @@ from typing import Any, Callable, Dict, Optional, Union
 
 import pkg_resources
 import yaml
+from typing_extensions import Literal
 
 from chia.util.path import mkdir
 
+PEER_DB_PATH_KEY_DEPRECATED = "peer_db_path"  # replaced by "peers_file_path"
+WALLET_PEERS_PATH_KEY_DEPRECATED = "wallet_peers_path"  # replaced by "wallet_peers_file_path"
+
 
 def initial_config_file(filename: Union[str, Path]) -> str:
     return pkg_resources.resource_string(__name__, f"initial-{filename}").decode()
@@ -159,3 +164,39 @@ def traverse_dict(d: Dict, key_path: str) -> Any:
         return val
     else:
         raise KeyError(f"value not found for key: {key}")
+
+
+method_strings = Literal["default", "python_default", "fork", "forkserver", "spawn"]
+method_values = Optional[Literal["fork", "forkserver", "spawn"]]
+start_methods: Dict[method_strings, method_values] = {
+    "default": None,
+    "python_default": None,
+    "fork": "fork",
+    "forkserver": "forkserver",
+    "spawn": "spawn",
+}
+
+
+def process_config_start_method(
+    config: Dict[str, Any],
+    log=logging.Logger,
+) -> method_values:
+    from_config: object = config.get("multiprocessing_start_method")
+
+    choice: method_strings
+    if from_config is None:
+        # handle not only the key being missing, but also set to None
+        choice = "default"
+    elif from_config not in start_methods.keys():
+        start_methods_string = ", ".join(option for option in start_methods.keys())
+        log.warning(f"Configured start method {from_config!r} not available in: {start_methods_string}")
+        choice = "default"
+    else:
+        # mypy doesn't realize that by the time we get here from_config must be one of
+        # the keys in `start_methods` due to the above `not in` condition.
+        choice = from_config  # type: ignore[assignment]
+
+    processed_method = start_methods[choice]
+    log.info(f"Selected multiprocessing start method: {choice}")
+
+    return processed_method
