@@ -1,4 +1,4 @@
-from typing import List, Tuple
+from typing import Any, Iterator, List, Tuple
 from chiabip158 import PyBIP158
 
 from chia.types.blockchain_format.coin import Coin
@@ -15,9 +15,15 @@ def get_block_header(block: FullBlock, tx_addition_coins: List[Coin], removals_n
     addition_coins = tx_addition_coins + list(block.get_included_reward_coins())
     if block.is_transaction_block():
         for coin in addition_coins:
-            byte_array_tx.append(bytearray(coin.puzzle_hash))
+            # TODO: address hint error and remove ignore
+            #       error: Argument 1 to "append" of "list" has incompatible type "bytearray"; expected "bytes32"
+            #       [arg-type]
+            byte_array_tx.append(bytearray(coin.puzzle_hash))  # type: ignore[arg-type]
         for name in removals_names:
-            byte_array_tx.append(bytearray(name))
+            # TODO: address hint error and remove ignore
+            #       error: Argument 1 to "append" of "list" has incompatible type "bytearray"; expected "bytes32"
+            #       [arg-type]
+            byte_array_tx.append(bytearray(name))  # type: ignore[arg-type]
 
     bip158: PyBIP158 = PyBIP158(byte_array_tx)
     encoded_filter: bytes = bytes(bip158.GetEncoded())
@@ -64,3 +70,14 @@ def tx_removals_and_additions(npc_list: List[NPC]) -> Tuple[List[bytes32], List[
     additions.extend(additions_for_npc(npc_list))
 
     return removals, additions
+
+
+def list_to_batches(list_to_split: List[Any], batch_size: int) -> Iterator[Tuple[int, List[Any]]]:
+    if batch_size <= 0:
+        raise ValueError("list_to_batches: batch_size must be greater than 0.")
+    total_size = len(list_to_split)
+    if total_size == 0:
+        return iter(())
+    for batch_start in range(0, total_size, batch_size):
+        batch_end = min(batch_start + batch_size, total_size)
+        yield total_size - batch_end, list_to_split[batch_start:batch_end]
