@@ -6,7 +6,7 @@ if [ -z "$VIRTUAL_ENV" ]; then
 	exit 1
 fi
 
-echo "Timelord requires CMake 3.14+ to compile vdf_client"
+echo "Timelord requires CMake 3.14+ to compile vdf_client."
 
 PYTHON_VERSION=$(python -c 'import sys; print(f"python{sys.version_info.major}.{sys.version_info.minor}")')
 echo "Python version: $PYTHON_VERSION"
@@ -18,7 +18,7 @@ CHIAVDF_VERSION=$(python -c 'from setup import dependencies; t = [_ for _ in dep
 ubuntu_cmake_install() {
 	UBUNTU_PRE_2004=$(python -c 'import subprocess; process = subprocess.run(["lsb_release", "-rs"], stdout=subprocess.PIPE); print(float(process.stdout) < float(20.04))')
 	if [ "$UBUNTU_PRE_2004" = "True" ]; then
-		echo "Ubuntu version is pre 20.04LTS - installing CMake with snap"
+		echo "Ubuntu version is pre 20.04LTS - installing CMake with snap."
 		sudo apt-get install snap -y
 		sudo apt-get remove --purge cmake -y
 		hash -r
@@ -36,19 +36,19 @@ symlink_vdf_bench() {
 	elif [ ! -e venv/lib/"$1"/site-packages/vdf_bench ]; then
 		echo "ERROR: Could not find venv/lib/$1/site-packages/vdf_bench"
 	else
-		echo "./vdf_bench link exists"
+		echo "./vdf_bench link exists."
 	fi
 }
 
 if [ "$(uname)" = "Linux" ] && type apt-get; then
 	UBUNTU_DEBIAN=true
-	echo "Found Ubuntu/Debian"
+	echo "Found Ubuntu/Debian."
 elif [ "$(uname)" = "Linux" ] && type dnf || yum; then
 	RHEL_BASED=true
-	echo "Found RedHat"
+	echo "Found RedHat."
 elif [ "$(uname)" = "Darwin" ]; then
 	MACOS=true
-	echo "Found MacOS"
+	echo "Found MacOS."
 fi
 
 if [ -e "$THE_PATH" ]; then
@@ -74,21 +74,21 @@ else
 		venv/bin/python -m pip install --force --no-binary chiavdf "$CHIAVDF_VERSION"
 		symlink_vdf_bench "$PYTHON_VERSION"
 	elif [ -e venv/bin/python ] && test $MACOS && [ "$(brew info boost | grep -c 'Not installed')" -eq 1 ]; then
-		echo "Installing chiavdf requirement boost for MacOS"
+		echo "Installing chiavdf requirement boost for MacOS."
 		brew install boost
-		echo "installing chiavdf from source"
+		echo "Installing chiavdf from source."
 		# User needs to provide required packages
 		echo venv/bin/python -m pip install --force --no-binary chiavdf "$CHIAVDF_VERSION"
 		venv/bin/python -m pip install --force --no-binary chiavdf "$CHIAVDF_VERSION"
 		symlink_vdf_bench "$PYTHON_VERSION"
 	elif [ -e venv/bin/python ]; then
-		echo "installing chiavdf from source"
+		echo "Installing chiavdf from source."
 		# User needs to provide required packages
 		echo venv/bin/python -m pip install --force --no-binary chiavdf "$CHIAVDF_VERSION"
 		venv/bin/python -m pip install --force --no-binary chiavdf "$CHIAVDF_VERSION"
 		symlink_vdf_bench "$PYTHON_VERSION"
 	else
-		echo "no venv created yet, please run install.sh"
+		echo "No venv created yet, please run install.sh."
 	fi
 fi
-echo "To estimate a timelord on this CPU try './vdf_bench square_asm 400000' for an ips estimate"
+echo "To estimate a timelord on this CPU try './vdf_bench square_asm 400000' for an ips estimate."
