@@ -2,9 +2,9 @@ import asyncio
 import logging
 
 import pytest
+import pytest_asyncio
 from clvm.casts import int_to_bytes
 
-from chia.consensus.blockchain import ReceiveBlockResult
 from chia.protocols import full_node_protocol, wallet_protocol
 from chia.types.announcement import Announcement
 from chia.types.condition_opcodes import ConditionOpcode
@@ -12,6 +12,7 @@ from chia.types.condition_with_args import ConditionWithArgs
 from chia.types.spend_bundle import SpendBundle
 from chia.util.errors import ConsensusError, Err
 from chia.util.ints import uint64
+from tests.blockchain.blockchain_test_utils import _validate_and_add_block
 from tests.wallet_tools import WalletTool
 from tests.setup_nodes import bt, setup_two_nodes, test_constants
 from tests.util.generator_tools_testing import run_and_get_removals_and_additions
@@ -31,9 +32,9 @@ def event_loop():
 
 
 class TestBlockchainTransactions:
-    @pytest.fixture(scope="function")
-    async def two_nodes(self):
-        async for _ in setup_two_nodes(test_constants):
+    @pytest_asyncio.fixture(scope="function")
+    async def two_nodes(self, db_version):
+        async for _ in setup_two_nodes(test_constants, db_version=db_version):
             yield _
 
     @pytest.mark.asyncio
@@ -133,9 +134,7 @@ class TestBlockchainTransactions:
         )
 
         next_block = new_blocks[-1]
-        res, err, _, _ = await full_node_1.blockchain.receive_block(next_block)
-        assert res == ReceiveBlockResult.INVALID_BLOCK
-        assert err == Err.DOUBLE_SPEND
+        await _validate_and_add_block(full_node_1.blockchain, next_block, expected_error=Err.DOUBLE_SPEND)
 
     @pytest.mark.asyncio
     async def test_validate_blockchain_duplicate_output(self, two_nodes):
@@ -173,9 +172,7 @@ class TestBlockchainTransactions:
         )
 
         next_block = new_blocks[-1]
-        res, err, _, _ = await full_node_1.blockchain.receive_block(next_block)
-        assert res == ReceiveBlockResult.INVALID_BLOCK
-        assert err == Err.DUPLICATE_OUTPUT
+        await _validate_and_add_block(full_node_1.blockchain, next_block, expected_error=Err.DUPLICATE_OUTPUT)
 
     @pytest.mark.asyncio
     async def test_validate_blockchain_with_reorg_double_spend(self, two_nodes):
@@ -232,9 +229,7 @@ class TestBlockchainTransactions:
             transaction_data=spend_bundle,
         )
 
-        res, err, _, _ = await full_node_api_1.full_node.blockchain.receive_block(new_blocks[-1])
-        assert err is None
-        assert res == ReceiveBlockResult.NEW_PEAK
+        await _validate_and_add_block(full_node_api_1.full_node.blockchain, new_blocks[-1])
 
         # But can't spend it twice
         new_blocks_double = bt.get_consecutive_blocks(
@@ -245,9 +240,9 @@ class TestBlockchainTransactions:
             transaction_data=spend_bundle,
         )
 
-        res, err, _, _ = await full_node_api_1.full_node.blockchain.receive_block(new_blocks_double[-1])
-        assert err is Err.DOUBLE_SPEND
-        assert res == ReceiveBlockResult.INVALID_BLOCK
+        await _validate_and_add_block(
+            full_node_api_1.full_node.blockchain, new_blocks_double[-1], expected_error=Err.DOUBLE_SPEND
+        )
 
         # Now test Reorg at block 5, same spend at block height 12
         new_blocks_reorg = bt.get_consecutive_blocks(
@@ -287,7 +282,7 @@ class TestBlockchainTransactions:
                 await full_node_api_1.full_node.respond_block(full_node_protocol.RespondBlock(block))
 
     @pytest.mark.asyncio
-    async def test_validate_blockchain_spend_reorg_coin(self, two_nodes):
+    async def test_validate_blockchain_spend_reorg_coin(self, two_nodes, softfork_height):
         num_blocks = 10
         wallet_a = WALLET_A
         coinbase_puzzlehash = WALLET_A_PUZZLE_HASHES[0]
@@ -326,7 +321,10 @@ class TestBlockchainTransactions:
 
         coin_2 = None
         for coin in run_and_get_removals_and_additions(
-            new_blocks[-1], test_constants.MAX_BLOCK_COST_CLVM, test_constants.COST_PER_BYTE
+            new_blocks[-1],
+            test_constants.MAX_BLOCK_COST_CLVM,
+            cost_per_byte=test_constants.COST_PER_BYTE,
+            height=softfork_height,
         )[1]:
             if coin.puzzle_hash == receiver_1_puzzlehash:
                 coin_2 = coin
@@ -347,7 +345,10 @@ class TestBlockchainTransactions:
 
         coin_3 = None
         for coin in run_and_get_removals_and_additions(
-            new_blocks[-1], test_constants.MAX_BLOCK_COST_CLVM, test_constants.COST_PER_BYTE
+            new_blocks[-1],
+            test_constants.MAX_BLOCK_COST_CLVM,
+            cost_per_byte=test_constants.COST_PER_BYTE,
+            height=softfork_height,
         )[1]:
             if coin.puzzle_hash == receiver_2_puzzlehash:
                 coin_3 = coin
@@ -511,9 +512,9 @@ class TestBlockchainTransactions:
         )
 
         # Try to validate that block
-        res, err, _, _ = await full_node_1.blockchain.receive_block(invalid_new_blocks[-1])
-        assert res == ReceiveBlockResult.INVALID_BLOCK
-        assert err == Err.ASSERT_MY_COIN_ID_FAILED
+        await _validate_and_add_block(
+            full_node_1.blockchain, invalid_new_blocks[-1], expected_error=Err.ASSERT_MY_COIN_ID_FAILED
+        )
 
         # Valid block bundle
         # Create another block that includes our transaction
@@ -524,9 +525,7 @@ class TestBlockchainTransactions:
             transaction_data=valid_spend_bundle,
             guarantee_transaction_block=True,
         )
-        res, err, _, _ = await full_node_1.blockchain.receive_block(new_blocks[-1])
-        assert res == ReceiveBlockResult.NEW_PEAK
-        assert err is None
+        await _validate_and_add_block(full_node_1.blockchain, new_blocks[-1])
 
     @pytest.mark.asyncio
     async def test_assert_coin_announcement_consumed(self, two_nodes):
@@ -591,9 +590,9 @@ class TestBlockchainTransactions:
         )
 
         # Try to validate that block
-        res, err, _, _ = await full_node_1.blockchain.receive_block(invalid_new_blocks[-1])
-        assert res == ReceiveBlockResult.INVALID_BLOCK
-        assert err == Err.ASSERT_ANNOUNCE_CONSUMED_FAILED
+        await _validate_and_add_block(
+            full_node_1.blockchain, invalid_new_blocks[-1], expected_error=Err.ASSERT_ANNOUNCE_CONSUMED_FAILED
+        )
 
         # bundle_together contains both transactions
         bundle_together = SpendBundle.aggregate([block1_spend_bundle, block2_spend_bundle])
@@ -608,9 +607,7 @@ class TestBlockchainTransactions:
         )
 
         # Try to validate newly created block
-        res, err, _, _ = await full_node_1.blockchain.receive_block(new_blocks[-1])
-        assert res == ReceiveBlockResult.NEW_PEAK
-        assert err is None
+        await _validate_and_add_block(full_node_1.blockchain, new_blocks[-1])
 
     @pytest.mark.asyncio
     async def test_assert_puzzle_announcement_consumed(self, two_nodes):
@@ -675,9 +672,9 @@ class TestBlockchainTransactions:
         )
 
         # Try to validate that block
-        res, err, _, _ = await full_node_1.blockchain.receive_block(invalid_new_blocks[-1])
-        assert res == ReceiveBlockResult.INVALID_BLOCK
-        assert err == Err.ASSERT_ANNOUNCE_CONSUMED_FAILED
+        await _validate_and_add_block(
+            full_node_1.blockchain, invalid_new_blocks[-1], expected_error=Err.ASSERT_ANNOUNCE_CONSUMED_FAILED
+        )
 
         # bundle_together contains both transactions
         bundle_together = SpendBundle.aggregate([block1_spend_bundle, block2_spend_bundle])
@@ -692,9 +689,7 @@ class TestBlockchainTransactions:
         )
 
         # Try to validate newly created block
-        res, err, _, _ = await full_node_1.blockchain.receive_block(new_blocks[-1])
-        assert res == ReceiveBlockResult.NEW_PEAK
-        assert err is None
+        await _validate_and_add_block(full_node_1.blockchain, new_blocks[-1])
 
     @pytest.mark.asyncio
     async def test_assert_height_absolute(self, two_nodes):
@@ -738,9 +733,9 @@ class TestBlockchainTransactions:
         )
 
         # Try to validate that block at index 10
-        res, err, _, _ = await full_node_1.blockchain.receive_block(invalid_new_blocks[-1])
-        assert res == ReceiveBlockResult.INVALID_BLOCK
-        assert err == Err.ASSERT_HEIGHT_ABSOLUTE_FAILED
+        await _validate_and_add_block(
+            full_node_1.blockchain, invalid_new_blocks[-1], expected_error=Err.ASSERT_HEIGHT_ABSOLUTE_FAILED
+        )
 
         new_blocks = bt.get_consecutive_blocks(
             1,
@@ -748,8 +743,7 @@ class TestBlockchainTransactions:
             farmer_reward_puzzle_hash=coinbase_puzzlehash,
             guarantee_transaction_block=True,
         )
-        res, _, _, _ = await full_node_1.blockchain.receive_block(new_blocks[-1])
-        assert res == ReceiveBlockResult.NEW_PEAK
+        await _validate_and_add_block(full_node_1.blockchain, new_blocks[-1])
 
         # At index 11, it can be spent
         new_blocks = bt.get_consecutive_blocks(
@@ -759,9 +753,7 @@ class TestBlockchainTransactions:
             transaction_data=block1_spend_bundle,
             guarantee_transaction_block=True,
         )
-        res, err, _, _ = await full_node_1.blockchain.receive_block(new_blocks[-1])
-        assert err is None
-        assert res == ReceiveBlockResult.NEW_PEAK
+        await _validate_and_add_block(full_node_1.blockchain, new_blocks[-1])
 
     @pytest.mark.asyncio
     async def test_assert_height_relative(self, two_nodes):
@@ -807,9 +799,9 @@ class TestBlockchainTransactions:
         )
 
         # Try to validate that block at index 11
-        res, err, _, _ = await full_node_1.blockchain.receive_block(invalid_new_blocks[-1])
-        assert res == ReceiveBlockResult.INVALID_BLOCK
-        assert err == Err.ASSERT_HEIGHT_RELATIVE_FAILED
+        await _validate_and_add_block(
+            full_node_1.blockchain, invalid_new_blocks[-1], expected_error=Err.ASSERT_HEIGHT_RELATIVE_FAILED
+        )
 
         new_blocks = bt.get_consecutive_blocks(
             1,
@@ -817,8 +809,7 @@ class TestBlockchainTransactions:
             farmer_reward_puzzle_hash=coinbase_puzzlehash,
             guarantee_transaction_block=True,
         )
-        res, _, _, _ = await full_node_1.blockchain.receive_block(new_blocks[-1])
-        assert res == ReceiveBlockResult.NEW_PEAK
+        await _validate_and_add_block(full_node_1.blockchain, new_blocks[-1])
 
         # At index 12, it can be spent
         new_blocks = bt.get_consecutive_blocks(
@@ -828,9 +819,7 @@ class TestBlockchainTransactions:
             transaction_data=block1_spend_bundle,
             guarantee_transaction_block=True,
         )
-        res, err, _, _ = await full_node_1.blockchain.receive_block(new_blocks[-1])
-        assert err is None
-        assert res == ReceiveBlockResult.NEW_PEAK
+        await _validate_and_add_block(full_node_1.blockchain, new_blocks[-1])
 
     @pytest.mark.asyncio
     async def test_assert_seconds_relative(self, two_nodes):
@@ -876,9 +865,9 @@ class TestBlockchainTransactions:
         )
 
         # Try to validate that block before 300 sec
-        res, err, _, _ = await full_node_1.blockchain.receive_block(invalid_new_blocks[-1])
-        assert res == ReceiveBlockResult.INVALID_BLOCK
-        assert err == Err.ASSERT_SECONDS_RELATIVE_FAILED
+        await _validate_and_add_block(
+            full_node_1.blockchain, invalid_new_blocks[-1], expected_error=Err.ASSERT_SECONDS_RELATIVE_FAILED
+        )
 
         valid_new_blocks = bt.get_consecutive_blocks(
             1,
@@ -888,9 +877,7 @@ class TestBlockchainTransactions:
             guarantee_transaction_block=True,
             time_per_block=301,
         )
-        res, err, _, _ = await full_node_1.blockchain.receive_block(valid_new_blocks[-1])
-        assert err is None
-        assert res == ReceiveBlockResult.NEW_PEAK
+        await _validate_and_add_block(full_node_1.blockchain, valid_new_blocks[-1])
 
     @pytest.mark.asyncio
     async def test_assert_seconds_absolute(self, two_nodes):
@@ -937,9 +924,9 @@ class TestBlockchainTransactions:
         )
 
         # Try to validate that block before 30 sec
-        res, err, _, _ = await full_node_1.blockchain.receive_block(invalid_new_blocks[-1])
-        assert res == ReceiveBlockResult.INVALID_BLOCK
-        assert err == Err.ASSERT_SECONDS_ABSOLUTE_FAILED
+        await _validate_and_add_block(
+            full_node_1.blockchain, invalid_new_blocks[-1], expected_error=Err.ASSERT_SECONDS_ABSOLUTE_FAILED
+        )
 
         valid_new_blocks = bt.get_consecutive_blocks(
             1,
@@ -949,9 +936,7 @@ class TestBlockchainTransactions:
             guarantee_transaction_block=True,
             time_per_block=31,
         )
-        res, err, _, _ = await full_node_1.blockchain.receive_block(valid_new_blocks[-1])
-        assert err is None
-        assert res == ReceiveBlockResult.NEW_PEAK
+        await _validate_and_add_block(full_node_1.blockchain, valid_new_blocks[-1])
 
     @pytest.mark.asyncio
     async def test_assert_fee_condition(self, two_nodes):
@@ -999,9 +984,9 @@ class TestBlockchainTransactions:
             guarantee_transaction_block=True,
         )
 
-        res, err, _, _ = await full_node_1.blockchain.receive_block(invalid_new_blocks[-1])
-        assert res == ReceiveBlockResult.INVALID_BLOCK
-        assert err == Err.RESERVE_FEE_CONDITION_FAILED
+        await _validate_and_add_block(
+            full_node_1.blockchain, invalid_new_blocks[-1], expected_error=Err.RESERVE_FEE_CONDITION_FAILED
+        )
 
         valid_new_blocks = bt.get_consecutive_blocks(
             1,
@@ -1010,6 +995,4 @@ class TestBlockchainTransactions:
             transaction_data=block1_spend_bundle_good,
             guarantee_transaction_block=True,
         )
-        res, err, _, _ = await full_node_1.blockchain.receive_block(valid_new_blocks[-1])
-        assert err is None
-        assert res == ReceiveBlockResult.NEW_PEAK
+        await _validate_and_add_block(full_node_1.blockchain, valid_new_blocks[-1])
