@@ -6,14 +6,18 @@ import random
 import time
 from secrets import token_bytes
 from typing import Dict, Optional, List
+from blspy import G2Element
 
+from clvm.casts import int_to_bytes
 import pytest
+import pytest_asyncio
 
+from chia.consensus.blockchain import ReceiveBlockResult
 from chia.consensus.pot_iterations import is_overflow_block
 from chia.full_node.bundle_tools import detect_potential_template_generator
 from chia.full_node.full_node_api import FullNodeAPI
 from chia.full_node.signage_point import SignagePoint
-from chia.protocols import full_node_protocol as fnp, full_node_protocol
+from chia.protocols import full_node_protocol as fnp, full_node_protocol, wallet_protocol
 from chia.protocols import timelord_protocol
 from chia.protocols.full_node_protocol import RespondTransaction
 from chia.protocols.protocol_message_types import ProtocolMessageTypes
@@ -21,8 +25,9 @@ from chia.server.address_manager import AddressManager
 from chia.server.outbound_message import Message
 from chia.simulator.simulator_protocol import FarmNewBlockProtocol
 from chia.types.blockchain_format.classgroup import ClassgroupElement
-from chia.types.blockchain_format.program import SerializedProgram
+from chia.types.blockchain_format.program import Program, SerializedProgram
 from chia.types.blockchain_format.vdf import CompressibleVDFField, VDFProof
+from chia.types.coin_spend import CoinSpend
 from chia.types.condition_opcodes import ConditionOpcode
 from chia.types.condition_with_args import ConditionWithArgs
 from chia.types.full_block import FullBlock
@@ -31,18 +36,22 @@ from chia.types.peer_info import PeerInfo, TimestampedPeerInfo
 from chia.types.spend_bundle import SpendBundle
 from chia.types.unfinished_block import UnfinishedBlock
 from tests.block_tools import get_signage_point
-from chia.util.clvm import int_to_bytes
 from chia.util.errors import Err
 from chia.util.hash import std_hash
 from chia.util.ints import uint8, uint16, uint32, uint64
 from chia.util.recursive_replace import recursive_replace
 from chia.util.vdf_prover import get_vdf_info_and_proof
+from tests.blockchain.blockchain_test_utils import (
+    _validate_and_add_block,
+    _validate_and_add_block_no_error,
+)
+from tests.pools.test_pool_rpc import wallet_is_synced
 from tests.wallet_tools import WalletTool
-from chia.wallet.cc_wallet.cc_wallet import CCWallet
+from chia.wallet.cat_wallet.cat_wallet import CATWallet
 from chia.wallet.transaction_record import TransactionRecord
 
 from tests.connection_utils import add_dummy_connection, connect_and_get_peer
-from tests.core.full_node.test_coin_store import get_future_reward_coins
+from tests.core.full_node.stores.test_coin_store import get_future_reward_coins
 from tests.core.full_node.test_mempool_performance import wallet_height_at_least
 from tests.core.make_block_generator import make_spend_bundle
 from tests.core.node_height import node_height_at_least
@@ -98,7 +107,7 @@ def event_loop():
     yield loop
 
 
-@pytest.fixture(scope="module")
+@pytest_asyncio.fixture(scope="module")
 async def wallet_nodes():
     async_gen = setup_simulators_and_wallets(2, 1, {"MEMPOOL_BLOCK_BUFFER": 2, "MAX_BLOCK_COST_CLVM": 400000000})
     nodes, wallets = await async_gen.__anext__()
@@ -114,27 +123,27 @@ async def wallet_nodes():
         yield _
 
 
-@pytest.fixture(scope="function")
-async def setup_four_nodes():
-    async for _ in setup_simulators_and_wallets(5, 0, {}, starting_port=51000):
+@pytest_asyncio.fixture(scope="function")
+async def setup_four_nodes(db_version):
+    async for _ in setup_simulators_and_wallets(5, 0, {}, db_version=db_version):
         yield _
 
 
-@pytest.fixture(scope="function")
-async def setup_two_nodes():
-    async for _ in setup_simulators_and_wallets(2, 0, {}, starting_port=51100):
+@pytest_asyncio.fixture(scope="function")
+async def setup_two_nodes(db_version):
+    async for _ in setup_simulators_and_wallets(2, 0, {}, db_version=db_version):
         yield _
 
 
-@pytest.fixture(scope="function")
+@pytest_asyncio.fixture(scope="function")
 async def setup_two_nodes_and_wallet():
-    async for _ in setup_simulators_and_wallets(2, 1, {}, starting_port=51200):
+    async for _ in setup_simulators_and_wallets(2, 1, {}, db_version=2):
         yield _
 
 
-@pytest.fixture(scope="function")
-async def wallet_nodes_mainnet():
-    async_gen = setup_simulators_and_wallets(2, 1, {"NETWORK_TYPE": 0}, starting_port=40000)
+@pytest_asyncio.fixture(scope="function")
+async def wallet_nodes_mainnet(db_version):
+    async_gen = setup_simulators_and_wallets(2, 1, {"NETWORK_TYPE": 0}, db_version=db_version)
     nodes, wallets = await async_gen.__anext__()
     full_node_1 = nodes[0]
     full_node_2 = nodes[1]
@@ -150,7 +159,8 @@ async def wallet_nodes_mainnet():
 
 class TestFullNodeBlockCompression:
     @pytest.mark.asyncio
-    async def do_test_block_compression(self, setup_two_nodes_and_wallet, empty_blockchain, tx_size, test_reorgs):
+    @pytest.mark.parametrize("tx_size", [10000, 3000000000000])
+    async def test_block_compression(self, setup_two_nodes_and_wallet, empty_blockchain, tx_size):
         nodes, wallets = setup_two_nodes_and_wallet
         server_1 = nodes[0].full_node.server
         server_2 = nodes[1].full_node.server
@@ -159,6 +169,14 @@ class TestFullNodeBlockCompression:
         full_node_2 = nodes[1]
         wallet_node_1 = wallets[0][0]
         wallet = wallet_node_1.wallet_state_manager.main_wallet
+
+        # Avoid retesting the slow reorg portion, not necessary more than once
+        test_reorgs = (
+            tx_size == 10000
+            and empty_blockchain.block_store.db_wrapper.db_version >= 2
+            and full_node_1.full_node.block_store.db_wrapper.db_version >= 2
+            and full_node_2.full_node.block_store.db_wrapper.db_version >= 2
+        )
         _ = await connect_and_get_peer(server_1, server_2)
         _ = await connect_and_get_peer(server_1, server_3)
 
@@ -167,9 +185,10 @@ class TestFullNodeBlockCompression:
         for i in range(4):
             await full_node_1.farm_new_transaction_block(FarmNewBlockProtocol(ph))
 
-        await time_out_assert(10, wallet_height_at_least, True, wallet_node_1, 4)
-        await time_out_assert(10, node_height_at_least, True, full_node_1, 4)
-        await time_out_assert(10, node_height_at_least, True, full_node_2, 4)
+        await time_out_assert(30, wallet_height_at_least, True, wallet_node_1, 4)
+        await time_out_assert(30, node_height_at_least, True, full_node_1, 4)
+        await time_out_assert(30, node_height_at_least, True, full_node_2, 4)
+        await time_out_assert(30, wallet_is_synced, True, wallet_node_1, full_node_1)
 
         # Send a transaction to mempool
         tr: TransactionRecord = await wallet.generate_signed_transaction(
@@ -186,9 +205,17 @@ class TestFullNodeBlockCompression:
 
         # Farm a block
         await full_node_1.farm_new_transaction_block(FarmNewBlockProtocol(ph))
-        await time_out_assert(10, node_height_at_least, True, full_node_1, 5)
-        await time_out_assert(10, node_height_at_least, True, full_node_2, 5)
-        await time_out_assert(10, wallet_height_at_least, True, wallet_node_1, 5)
+        await time_out_assert(30, node_height_at_least, True, full_node_1, 5)
+        await time_out_assert(30, node_height_at_least, True, full_node_2, 5)
+        await time_out_assert(30, wallet_height_at_least, True, wallet_node_1, 5)
+        await time_out_assert(30, wallet_is_synced, True, wallet_node_1, full_node_1)
+
+        async def check_transaction_confirmed(transaction) -> bool:
+            tx = await wallet_node_1.wallet_state_manager.get_transaction(transaction.name)
+            return tx.confirmed
+
+        await time_out_assert(30, check_transaction_confirmed, True, tr)
+        await asyncio.sleep(2)
 
         # Confirm generator is not compressed
         program: Optional[SerializedProgram] = (await full_node_1.get_all_full_blocks())[-1].transactions_generator
@@ -214,6 +241,10 @@ class TestFullNodeBlockCompression:
         await time_out_assert(10, node_height_at_least, True, full_node_1, 6)
         await time_out_assert(10, node_height_at_least, True, full_node_2, 6)
         await time_out_assert(10, wallet_height_at_least, True, wallet_node_1, 6)
+        await time_out_assert(30, wallet_is_synced, True, wallet_node_1, full_node_1)
+
+        await time_out_assert(10, check_transaction_confirmed, True, tr)
+        await asyncio.sleep(2)
 
         # Confirm generator is compressed
         program: Optional[SerializedProgram] = (await full_node_1.get_all_full_blocks())[-1].transactions_generator
@@ -227,6 +258,7 @@ class TestFullNodeBlockCompression:
         await time_out_assert(10, node_height_at_least, True, full_node_1, 8)
         await time_out_assert(10, node_height_at_least, True, full_node_2, 8)
         await time_out_assert(10, wallet_height_at_least, True, wallet_node_1, 8)
+        await time_out_assert(30, wallet_is_synced, True, wallet_node_1, full_node_1)
 
         # Send another 2 tx
         tr: TransactionRecord = await wallet.generate_signed_transaction(
@@ -281,6 +313,10 @@ class TestFullNodeBlockCompression:
         await time_out_assert(10, node_height_at_least, True, full_node_1, 9)
         await time_out_assert(10, node_height_at_least, True, full_node_2, 9)
         await time_out_assert(10, wallet_height_at_least, True, wallet_node_1, 9)
+        await time_out_assert(30, wallet_is_synced, True, wallet_node_1, full_node_1)
+
+        await time_out_assert(10, check_transaction_confirmed, True, tr)
+        await asyncio.sleep(2)
 
         # Confirm generator is compressed
         program: Optional[SerializedProgram] = (await full_node_1.get_all_full_blocks())[-1].transactions_generator
@@ -288,27 +324,34 @@ class TestFullNodeBlockCompression:
         assert detect_potential_template_generator(uint32(9), program) is None
         assert len((await full_node_1.get_all_full_blocks())[-1].transactions_generator_ref_list) > 0
 
-        # Creates a cc wallet
-        cc_wallet: CCWallet = await CCWallet.create_new_cc(wallet_node_1.wallet_state_manager, wallet, uint64(100))
-        tx_queue: List[TransactionRecord] = await wallet_node_1.wallet_state_manager.tx_store.get_not_sent()
-        tr = tx_queue[0]
-        await time_out_assert(
-            10,
-            full_node_1.full_node.mempool_manager.get_spendbundle,
-            tr.spend_bundle,
-            tr.spend_bundle.name(),
-        )
-
+        # Creates a standard_transaction and an anyone-can-spend tx
         tr: TransactionRecord = await wallet.generate_signed_transaction(
             30000,
-            ph,
+            Program.to(1).get_tree_hash(),
         )
-        await wallet.push_transaction(tx=tr)
+        extra_spend = SpendBundle(
+            [
+                CoinSpend(
+                    next(coin for coin in tr.additions if coin.puzzle_hash == Program.to(1).get_tree_hash()),
+                    Program.to(1),
+                    Program.to([[51, ph, 30000]]),
+                )
+            ],
+            G2Element(),
+        )
+        new_spend_bundle = SpendBundle.aggregate([tr.spend_bundle, extra_spend])
+        new_tr = dataclasses.replace(
+            tr,
+            spend_bundle=new_spend_bundle,
+            additions=new_spend_bundle.additions(),
+            removals=new_spend_bundle.removals(),
+        )
+        await wallet.push_transaction(tx=new_tr)
         await time_out_assert(
             10,
             full_node_2.full_node.mempool_manager.get_spendbundle,
-            tr.spend_bundle,
-            tr.name,
+            new_tr.spend_bundle,
+            new_tr.spend_bundle.name(),
         )
 
         # Farm a block
@@ -316,33 +359,45 @@ class TestFullNodeBlockCompression:
         await time_out_assert(10, node_height_at_least, True, full_node_1, 10)
         await time_out_assert(10, node_height_at_least, True, full_node_2, 10)
         await time_out_assert(10, wallet_height_at_least, True, wallet_node_1, 10)
+        await time_out_assert(30, wallet_is_synced, True, wallet_node_1, full_node_1)
 
-        # Confirm generator is compressed
-        program: Optional[SerializedProgram] = (await full_node_1.get_all_full_blocks())[-1].transactions_generator
+        await time_out_assert(10, check_transaction_confirmed, True, new_tr)
+        await asyncio.sleep(2)
+
+        # Confirm generator is not compressed, #CAT creation has a cat spend
+        all_blocks = await full_node_1.get_all_full_blocks()
+        program: Optional[SerializedProgram] = all_blocks[-1].transactions_generator
         assert program is not None
-        assert detect_potential_template_generator(uint32(10), program) is None
-        assert len((await full_node_1.get_all_full_blocks())[-1].transactions_generator_ref_list) > 0
+        assert len(all_blocks[-1].transactions_generator_ref_list) == 0
 
-        # Make a cc transaction
-        tr: TransactionRecord = await cc_wallet.generate_signed_transaction([uint64(60)], [ph])
-        await wallet.wallet_state_manager.add_pending_transaction(tr)
-        await time_out_assert(
-            10,
-            full_node_2.full_node.mempool_manager.get_spendbundle,
-            tr.spend_bundle,
-            tr.name,
-        )
-        # Make a standard transaction
+        # Make a standard transaction and an anyone-can-spend transaction
         tr: TransactionRecord = await wallet.generate_signed_transaction(
             30000,
-            ph,
+            Program.to(1).get_tree_hash(),
         )
-        await wallet.push_transaction(tx=tr)
+        extra_spend = SpendBundle(
+            [
+                CoinSpend(
+                    next(coin for coin in tr.additions if coin.puzzle_hash == Program.to(1).get_tree_hash()),
+                    Program.to(1),
+                    Program.to([[51, ph, 30000]]),
+                )
+            ],
+            G2Element(),
+        )
+        new_spend_bundle = SpendBundle.aggregate([tr.spend_bundle, extra_spend])
+        new_tr = dataclasses.replace(
+            tr,
+            spend_bundle=new_spend_bundle,
+            additions=new_spend_bundle.additions(),
+            removals=new_spend_bundle.removals(),
+        )
+        await wallet.push_transaction(tx=new_tr)
         await time_out_assert(
             10,
             full_node_2.full_node.mempool_manager.get_spendbundle,
-            tr.spend_bundle,
-            tr.name,
+            new_tr.spend_bundle,
+            new_tr.spend_bundle.name(),
         )
 
         # Farm a block
@@ -350,6 +405,7 @@ class TestFullNodeBlockCompression:
         await time_out_assert(10, node_height_at_least, True, full_node_1, 11)
         await time_out_assert(10, node_height_at_least, True, full_node_2, 11)
         await time_out_assert(10, wallet_height_at_least, True, wallet_node_1, 11)
+        await time_out_assert(30, wallet_is_synced, True, wallet_node_1, full_node_1)
 
         # Confirm generator is not compressed
         program: Optional[SerializedProgram] = (await full_node_1.get_all_full_blocks())[-1].transactions_generator
@@ -368,20 +424,24 @@ class TestFullNodeBlockCompression:
             reog_blocks = bt.get_consecutive_blocks(14)
             for r in range(0, len(reog_blocks), 3):
                 for reorg_block in reog_blocks[:r]:
-                    assert (await blockchain.receive_block(reorg_block))[1] is None
+                    await _validate_and_add_block_no_error(blockchain, reorg_block)
                 for i in range(1, height):
                     for batch_size in range(1, height):
-                        results = await blockchain.pre_validate_blocks_multiprocessing(all_blocks[:i], {}, batch_size)
+                        results = await blockchain.pre_validate_blocks_multiprocessing(
+                            all_blocks[:i], {}, batch_size, validate_signatures=False
+                        )
                         assert results is not None
                         for result in results:
                             assert result.error is None
 
             for r in range(0, len(all_blocks), 3):
                 for block in all_blocks[:r]:
-                    assert (await blockchain.receive_block(block))[1] is None
+                    await _validate_and_add_block_no_error(blockchain, block)
                 for i in range(1, height):
                     for batch_size in range(1, height):
-                        results = await blockchain.pre_validate_blocks_multiprocessing(all_blocks[:i], {}, batch_size)
+                        results = await blockchain.pre_validate_blocks_multiprocessing(
+                            all_blocks[:i], {}, batch_size, validate_signatures=False
+                        )
                         assert results is not None
                         for result in results:
                             assert result.error is None
@@ -391,14 +451,6 @@ class TestFullNodeBlockCompression:
                 await full_node_1.full_node.respond_block(full_node_protocol.RespondBlock(block))
             assert full_node_1.full_node.full_node_store.previous_generator is None
 
-    @pytest.mark.asyncio
-    async def test_block_compression(self, setup_two_nodes_and_wallet, empty_blockchain):
-        await self.do_test_block_compression(setup_two_nodes_and_wallet, empty_blockchain, 10000, True)
-
-    @pytest.mark.asyncio
-    async def test_block_compression_2(self, setup_two_nodes_and_wallet, empty_blockchain):
-        await self.do_test_block_compression(setup_two_nodes_and_wallet, empty_blockchain, 3000000000000, False)
-
 
 class TestFullNodeProtocol:
     @pytest.mark.asyncio
@@ -728,7 +780,7 @@ class TestFullNodeProtocol:
         assert full_node_1.full_node.full_node_store.get_unfinished_block(unf.partial_hash) is not None
         result = full_node_1.full_node.full_node_store.get_unfinished_block_result(unf.partial_hash)
         assert result is not None
-        assert result.npc_result is not None and result.npc_result.clvm_cost > 0
+        assert result.npc_result is not None and result.npc_result.cost > 0
 
         assert not full_node_1.full_node.blockchain.contains_block(block.header_hash)
         assert block.transactions_generator is not None
@@ -762,13 +814,15 @@ class TestFullNodeProtocol:
                 uint32(0),
                 block.reward_chain_block.get_unfinished().get_hash(),
             )
-            asyncio.create_task(full_node_1.new_peak(new_peak, dummy_peer))
+            task_1 = asyncio.create_task(full_node_1.new_peak(new_peak, dummy_peer))
             await time_out_assert(10, time_out_messages(incoming_queue, "request_block", 1))
+            task_1.cancel()
 
             await full_node_1.full_node.respond_block(fnp.RespondBlock(block), peer)
             # Ignores, already have
-            asyncio.create_task(full_node_1.new_peak(new_peak, dummy_peer))
+            task_2 = asyncio.create_task(full_node_1.new_peak(new_peak, dummy_peer))
             await time_out_assert(10, time_out_messages(incoming_queue, "request_block", 0))
+            task_2.cancel()
 
         # Ignores low weight
         new_peak = fnp.NewPeak(
@@ -840,9 +894,9 @@ class TestFullNodeProtocol:
             )
             assert spend_bundle is not None
             cost_result = await full_node_1.full_node.mempool_manager.pre_validate_spendbundle(
-                spend_bundle, spend_bundle.name()
+                spend_bundle, None, spend_bundle.name()
             )
-            log.info(f"Cost result: {cost_result.clvm_cost}")
+            log.info(f"Cost result: {cost_result.cost}")
 
             new_transaction = fnp.NewTransaction(spend_bundle.get_hash(), uint64(100), uint64(100))
 
@@ -886,9 +940,9 @@ class TestFullNodeProtocol:
             spend_bundle = wallet_receiver.generate_signed_transaction(
                 uint64(500), receiver_puzzlehash, coin_record.coin, fee=fee
             )
-            respond_transaction = fnp.RespondTransaction(spend_bundle)
+            respond_transaction = wallet_protocol.SendTransaction(spend_bundle)
 
-            await full_node_1.respond_transaction(respond_transaction, peer)
+            await full_node_1.send_transaction(respond_transaction)
 
             request = fnp.RequestTransaction(spend_bundle.get_hash())
             req = await full_node_1.request_transaction(request)
@@ -1348,7 +1402,7 @@ class TestFullNodeProtocol:
 
         second_blockchain = empty_blockchain
         for block in blocks:
-            await second_blockchain.receive_block(block)
+            await _validate_and_add_block(second_blockchain, block)
 
         # Creates a signage point based on the last block
         peak_2 = second_blockchain.get_peak()
@@ -1454,9 +1508,9 @@ class TestFullNodeProtocol:
         invalid_program = SerializedProgram.from_bytes(large_puzzle_reveal)
         invalid_block = dataclasses.replace(invalid_block, transactions_generator=invalid_program)
 
-        result, error, fork_h, _ = await full_node_1.full_node.blockchain.receive_block(invalid_block)
-        assert error is not None
-        assert error == Err.PRE_SOFT_FORK_MAX_GENERATOR_SIZE
+        await _validate_and_add_block(
+            full_node_1.full_node.blockchain, invalid_block, expected_error=Err.PRE_SOFT_FORK_MAX_GENERATOR_SIZE
+        )
 
         blocks_new = bt.get_consecutive_blocks(
             1,
@@ -1467,18 +1521,17 @@ class TestFullNodeProtocol:
         valid_block = blocks_new[-1]
         valid_program = SerializedProgram.from_bytes(under_sized)
         valid_block = dataclasses.replace(valid_block, transactions_generator=valid_program)
-        result, error, fork_h = await full_node_1.full_node.blockchain.receive_block(valid_block)
-
-        assert error is None
+        await _validate_and_add_block(full_node_1.full_node.blockchain, valid_block)
 
     @pytest.mark.asyncio
     async def test_compact_protocol(self, setup_two_nodes):
         nodes, _ = setup_two_nodes
         full_node_1 = nodes[0]
         full_node_2 = nodes[1]
-        blocks = bt.get_consecutive_blocks(num_blocks=1, skip_slots=3)
+        blocks = bt.get_consecutive_blocks(num_blocks=10, skip_slots=3)
         block = blocks[0]
-        await full_node_1.full_node.respond_block(fnp.RespondBlock(block))
+        for b in blocks:
+            await full_node_1.full_node.respond_block(fnp.RespondBlock(b))
         timelord_protocol_finished = []
         cc_eos_count = 0
         for sub_slot in block.finished_sub_slots:
@@ -1499,9 +1552,10 @@ class TestFullNodeProtocol:
                     CompressibleVDFField.CC_EOS_VDF,
                 )
             )
-        blocks_2 = bt.get_consecutive_blocks(num_blocks=2, block_list_input=blocks, skip_slots=3)
-        block = blocks_2[1]
-        await full_node_1.full_node.respond_block(fnp.RespondBlock(block))
+        blocks_2 = bt.get_consecutive_blocks(num_blocks=10, block_list_input=blocks, skip_slots=3)
+        block = blocks_2[-10]
+        for b in blocks_2[-11:]:
+            await full_node_1.full_node.respond_block(fnp.RespondBlock(b))
         icc_eos_count = 0
         for sub_slot in block.finished_sub_slots:
             if sub_slot.infused_challenge_chain is not None:
