@@ -1,7 +1,9 @@
 # flake8: noqa: F811, F401
 import logging
+from typing import List
 
 import pytest
+import pytest_asyncio
 from blspy import AugSchemeMPL
 
 from chia.consensus.pot_iterations import is_overflow_block
@@ -10,20 +12,24 @@ from chia.protocols import full_node_protocol
 from chia.rpc.full_node_rpc_api import FullNodeRpcApi
 from chia.rpc.full_node_rpc_client import FullNodeRpcClient
 from chia.rpc.rpc_server import NodeType, start_rpc_server
-from chia.simulator.simulator_protocol import FarmNewBlockProtocol
+from chia.simulator.simulator_protocol import FarmNewBlockProtocol, ReorgProtocol
+from chia.types.full_block import FullBlock
 from chia.types.spend_bundle import SpendBundle
 from chia.types.unfinished_block import UnfinishedBlock
 from tests.block_tools import get_signage_point
 from chia.util.hash import std_hash
 from chia.util.ints import uint16, uint8
+from tests.blockchain.blockchain_test_utils import _validate_and_add_block
 from tests.wallet_tools import WalletTool
 from tests.connection_utils import connect_and_get_peer
 from tests.setup_nodes import bt, self_hostname, setup_simulators_and_wallets, test_constants
 from tests.time_out_assert import time_out_assert
+from tests.util.rpc import validate_get_routes
+from tests.util.socket import find_available_listen_port
 
 
 class TestRpc:
-    @pytest.fixture(scope="function")
+    @pytest_asyncio.fixture(scope="function")
     async def two_nodes(self):
         async for _ in setup_simulators_and_wallets(2, 0, {}):
             yield _
@@ -31,7 +37,7 @@ class TestRpc:
     @pytest.mark.asyncio
     async def test1(self, two_nodes):
         num_blocks = 5
-        test_rpc_port = uint16(21522)
+        test_rpc_port = find_available_listen_port()
         nodes, _ = two_nodes
         full_node_api_1, full_node_api_2 = nodes
         server_1 = full_node_api_1.full_node.server
@@ -60,6 +66,7 @@ class TestRpc:
 
         try:
             client = await FullNodeRpcClient.create(self_hostname, test_rpc_port, bt.root_path, config)
+            await validate_get_routes(client, full_node_rpc_api)
             state = await client.get_blockchain_state()
             assert state["peak"] is None
             assert not state["sync"]["sync_mode"]
@@ -202,6 +209,21 @@ class TestRpc:
             assert len(await client.get_connections(NodeType.FARMER)) == 0
             await client.close_connection(connections[0]["node_id"])
             await time_out_assert(10, num_connections, 0)
+
+            blocks: List[FullBlock] = await client.get_blocks(0, 5)
+            assert len(blocks) == 5
+
+            await full_node_api_1.reorg_from_index_to_new_index(ReorgProtocol(2, 55, bytes([0x2] * 32)))
+            new_blocks_0: List[FullBlock] = await client.get_blocks(0, 5)
+            assert len(new_blocks_0) == 7
+
+            new_blocks: List[FullBlock] = await client.get_blocks(0, 5, exclude_reorged=True)
+            assert len(new_blocks) == 5
+            assert blocks[0].header_hash == new_blocks[0].header_hash
+            assert blocks[1].header_hash == new_blocks[1].header_hash
+            assert blocks[2].header_hash == new_blocks[2].header_hash
+            assert blocks[3].header_hash != new_blocks[3].header_hash
+
         finally:
             # Checks that the RPC manages to stop the node
             client.close()
@@ -210,7 +232,7 @@ class TestRpc:
 
     @pytest.mark.asyncio
     async def test_signage_points(self, two_nodes, empty_blockchain):
-        test_rpc_port = uint16(21522)
+        test_rpc_port = find_available_listen_port()
         nodes, _ = two_nodes
         full_node_api_1, full_node_api_2 = nodes
         server_1 = full_node_api_1.full_node.server
@@ -264,7 +286,7 @@ class TestRpc:
             second_blockchain = empty_blockchain
 
             for block in blocks:
-                await second_blockchain.receive_block(block)
+                await _validate_and_add_block(second_blockchain, block)
 
             # Creates a signage point based on the last block
             peak_2 = second_blockchain.get_peak()
