@@ -89,6 +89,49 @@ def child_writer_dispatch(func, lock_path: Path, timeout: int, max_iters: int):
         raise e
 
 
+def child_writer_dispatch_with_readiness_check(
+    func, lock_path: Path, timeout: int, max_iters: int, ready_dir: Path, finished_dir: Path
+):
+    # Write out a file indicating this process is ready to begin
+    ready_file_path: Path = ready_dir / f"{os.getpid()}.ready"
+    with open(ready_file_path, "w") as f:
+        f.write(f"{os.getpid()}\n")
+
+    # Wait up to 30 seconds for all processes to indicate readiness
+    start_file_path: Path = ready_dir / "start"
+    remaining_attempts = 120
+    while remaining_attempts > 0:
+        if start_file_path.exists():
+            break
+        else:
+            sleep(0.25)
+            remaining_attempts -= 1
+
+    assert remaining_attempts >= 0
+
+    try:
+        with acquire_writer_lock(lock_path, timeout, max_iters):
+            result = func()
+            return result
+    except FileKeyringLockTimeout as e:
+        log.warning(
+            f"[pid:{os.getpid()}] caught exception in child_writer_dispatch_with_readiness_check: "
+            f"FileKeyringLockTimeout {e}"
+        )
+        raise e
+    except Exception as e:
+        log.warning(
+            f"[pid:{os.getpid()}] caught exception in child_writer_dispatch_with_readiness_check: "
+            f"type: {type(e)}, {e}"
+        )
+        raise e
+    finally:
+        # Write out a file indicating this process has completed its work
+        finished_file_path: Path = finished_dir / f"{os.getpid()}.finished"
+        with open(finished_file_path, "w") as f:
+            f.write(f"{os.getpid()}\n")
+
+
 def poll_directory(dir: Path, expected_entries: int, max_attempts: int, interval: float = 1.0) -> bool:
     found_all: bool = False
     remaining_attempts: int = 30
@@ -105,6 +148,17 @@ def poll_directory(dir: Path, expected_entries: int, max_attempts: int, interval
 
 
 class TestFileKeyringSynchronization:
+    @pytest.fixture(scope="function")
+    def ready_dir(self, tmp_path: Path):
+        ready_dir: Path = tmp_path / "ready"
+        mkdir(ready_dir)
+        return ready_dir
+
+    @pytest.fixture(scope="function")
+    def finished_dir(self, tmp_path: Path):
+        finished_dir: Path = tmp_path / "finished"
+        mkdir(finished_dir)
+        return finished_dir
 
     # When: using a new empty keyring
     @using_temp_file_keyring()
@@ -155,7 +209,7 @@ class TestFileKeyringSynchronization:
 
     # When: using a new empty keyring
     @using_temp_file_keyring()
-    def test_writer_lock_timeout(self):
+    def test_writer_lock_timeout(self, ready_dir: Path, finished_dir: Path):
         """
         If a writer lock is already held, another process should not be able to acquire
         the same lock, failing after n attempts
@@ -169,10 +223,29 @@ class TestFileKeyringSynchronization:
         child_proc_fn = dummy_fn_requiring_writer_lock
         timeout = 0.25
         attempts = 4
+        num_workers = 1
 
-        with Pool(processes=1) as pool:
+        with Pool(processes=num_workers) as pool:
             # When: a child process attempts to acquire the same writer lock, failing after 1 second
-            res = pool.starmap_async(child_writer_dispatch, [(child_proc_fn, lock_path, timeout, attempts)])
+            res = pool.starmap_async(
+                child_writer_dispatch_with_readiness_check,
+                [(child_proc_fn, lock_path, timeout, attempts, ready_dir, finished_dir)],
+            )
+
+            # Wait up to 30 seconds for all processes to indicate readiness
+            assert poll_directory(ready_dir, num_workers, 30) is True
+
+            log.warning(f"Test setup complete: {num_workers} workers ready")
+
+            # Signal that testing should begin
+            start_file_path: Path = ready_dir / "start"
+            with open(start_file_path, "w") as f:
+                f.write(f"{os.getpid()}\n")
+
+            # Wait up to 30 seconds for all processes to indicate completion
+            assert poll_directory(finished_dir, num_workers, 30) is True
+
+            log.warning(f"Finished: {num_workers} workers finished")
 
             # Expect: the child to fail acquiring the writer lock (raises as FileKeyringLockTimeout)
             with pytest.raises(FileKeyringLockTimeout):
@@ -183,7 +256,7 @@ class TestFileKeyringSynchronization:
 
     # When: using a new empty keyring
     @using_temp_file_keyring()
-    def test_writer_lock_succeeds(self):
+    def test_writer_lock_succeeds(self, ready_dir: Path, finished_dir: Path):
         """
         If a write lock is already held, another process will be able to acquire the
         same lock once the lock is released by the current holder
@@ -196,14 +269,28 @@ class TestFileKeyringSynchronization:
 
         child_proc_fn = dummy_fn_requiring_writer_lock
         timeout = 0.25
-        attempts = 4
+        attempts = 8
+        num_workers = 1
 
-        with Pool(processes=1) as pool:
+        with Pool(processes=num_workers) as pool:
             # When: a child process attempts to acquire the same writer lock, failing after 1 second
-            res = pool.starmap_async(child_writer_dispatch, [(child_proc_fn, lock_path, timeout, attempts)])
+            res = pool.starmap_async(
+                child_writer_dispatch_with_readiness_check,
+                [(child_proc_fn, lock_path, timeout, attempts, ready_dir, finished_dir)],
+            )
+
+            # Wait up to 30 seconds for all processes to indicate readiness
+            assert poll_directory(ready_dir, num_workers, 30) is True
+
+            log.warning(f"Test setup complete: {num_workers} workers ready")
+
+            # Signal that testing should begin
+            start_file_path: Path = ready_dir / "start"
+            with open(start_file_path, "w") as f:
+                f.write(f"{os.getpid()}\n")
 
             # Brief delay to allow the child to timeout once
-            sleep(0.25)
+            sleep(0.50)
 
             # When: the writer lock is released
             lock.release_write_lock()
@@ -212,9 +299,14 @@ class TestFileKeyringSynchronization:
             result = res.get(timeout=10)  # 10 second timeout to prevent a bad test from spoiling the fun
             assert result[0] == "A winner is you!"
 
+            # Wait up to 30 seconds for all processes to indicate completion
+            assert poll_directory(finished_dir, num_workers, 30) is True
+
+            log.warning(f"Finished: {num_workers} workers finished")
+
     # When: using a new empty keyring
     @using_temp_file_keyring()
-    def test_writer_lock_reacquisition_failure(self):
+    def test_writer_lock_reacquisition_failure(self, ready_dir: Path, finished_dir: Path):
         """
         After the child process acquires the writer lock (and sleeps), the previous
         holder should not be able to quickly reacquire the lock
@@ -228,10 +320,24 @@ class TestFileKeyringSynchronization:
         child_proc_function = dummy_sleep_fn  # Sleeps for DUMMY_SLEEP_VALUE seconds
         timeout = 0.25
         attempts = 8
+        num_workers = 1
 
-        with Pool(processes=1) as pool:
+        with Pool(processes=num_workers) as pool:
             # When: a child process attempts to acquire the same writer lock, failing after 1 second
-            pool.starmap_async(child_writer_dispatch, [(child_proc_function, lock_path, timeout, attempts)])
+            pool.starmap_async(
+                child_writer_dispatch_with_readiness_check,
+                [(child_proc_function, lock_path, timeout, attempts, ready_dir, finished_dir)],
+            )
+
+            # Wait up to 30 seconds for all processes to indicate readiness
+            assert poll_directory(ready_dir, num_workers, 30) is True
+
+            log.warning(f"Test setup complete: {num_workers} workers ready")
+
+            # Signal that testing should begin
+            start_file_path: Path = ready_dir / "start"
+            with open(start_file_path, "w") as f:
+                f.write(f"{os.getpid()}\n")
 
             # When: the writer lock is released
             lock.release_write_lock()
@@ -242,9 +348,14 @@ class TestFileKeyringSynchronization:
             # Expect: Reacquiring the lock should fail due to the child holding the lock and sleeping
             assert lock.acquire_write_lock(timeout=0.25) is False
 
+            # Wait up to 30 seconds for all processes to indicate completion
+            assert poll_directory(finished_dir, num_workers, 30) is True
+
+            log.warning(f"Finished: {num_workers} workers finished")
+
     # When: using a new empty keyring
     @using_temp_file_keyring()
-    def test_writer_lock_reacquisition_success(self):
+    def test_writer_lock_reacquisition_success(self, ready_dir: Path, finished_dir: Path):
         """
         After the child process releases the writer lock, we should be able to
         acquire the lock
@@ -258,28 +369,44 @@ class TestFileKeyringSynchronization:
         child_proc_function = dummy_sleep_fn  # Sleeps for DUMMY_SLEEP_VALUE seconds
         timeout = 0.25
         attempts = 4
+        num_workers = 1
 
-        with Pool(processes=1) as pool:
+        with Pool(processes=num_workers) as pool:
             # When: a child process attempts to acquire the same writer lock, failing after 1 second
-            pool.starmap_async(child_writer_dispatch, [(child_proc_function, lock_path, timeout, attempts)])
+            pool.starmap_async(
+                child_writer_dispatch_with_readiness_check,
+                [(child_proc_function, lock_path, timeout, attempts, ready_dir, finished_dir)],
+            )
+
+            # Wait up to 30 seconds for all processes to indicate readiness
+            assert poll_directory(ready_dir, num_workers, 30) is True
+
+            log.warning(f"Test setup complete: {num_workers} workers ready")
+
+            # Signal that testing should begin
+            start_file_path: Path = ready_dir / "start"
+            with open(start_file_path, "w") as f:
+                f.write(f"{os.getpid()}\n")
 
             # When: the writer lock is released
             lock.release_write_lock()
 
+            # Wait up to 30 seconds for all processes to indicate completion
+            assert poll_directory(finished_dir, num_workers, 30) is True
+
+            log.warning(f"Finished: {num_workers} workers finished")
+
             # Expect: Reacquiring the lock should succeed after the child finishes and releases the lock
             assert lock.acquire_write_lock(timeout=(DUMMY_SLEEP_VALUE + 0.25)) is True
 
     # When: using a new empty keyring
+    @pytest.mark.skipif(platform == "darwin", reason="triggers the CrashReporter prompt")
     @using_temp_file_keyring()
     def test_writer_lock_released_on_abort(self):
         """
         When a child process is holding the lock and aborts/crashes, we should be
         able to acquire the lock
         """
-        # Avoid running on macOS: calling abort() triggers the CrashReporter prompt, interfering with automated testing
-        if platform == "darwin":
-            return
-
         lock_path = FileKeyring.lockfile_path_for_file_path(KeyringWrapper.get_shared_instance().keyring.keyring_path)
         lock = fasteners.InterProcessReaderWriterLock(str(lock_path))
 
@@ -306,7 +433,7 @@ class TestFileKeyringSynchronization:
 
     # When: using a new empty keyring
     @using_temp_file_keyring()
-    def test_writer_lock_blocked_by_readers(self):
+    def test_writer_lock_blocked_by_readers(self, ready_dir: Path, finished_dir: Path):
         """
         When a reader lock is already held, another thread/process should not be able
         to acquire the lock for writing
@@ -320,20 +447,39 @@ class TestFileKeyringSynchronization:
         child_proc_function = dummy_fn_requiring_writer_lock
         timeout = 0.25
         attempts = 4
+        num_workers = 1
 
-        with Pool(processes=1) as pool:
+        with Pool(processes=num_workers) as pool:
             # When: a child process attempts to acquire the same lock for writing, failing after 1 second
-            res = pool.starmap_async(child_writer_dispatch, [(child_proc_function, lock_path, timeout, attempts)])
+            res = pool.starmap_async(
+                child_writer_dispatch_with_readiness_check,
+                [(child_proc_function, lock_path, timeout, attempts, ready_dir, finished_dir)],
+            )
+
+            # Wait up to 30 seconds for all processes to indicate readiness
+            assert poll_directory(ready_dir, num_workers, 30) is True
+
+            log.warning(f"Test setup complete: {num_workers} workers ready")
+
+            # Signal that testing should begin
+            start_file_path: Path = ready_dir / "start"
+            with open(start_file_path, "w") as f:
+                f.write(f"{os.getpid()}\n")
+
+            # Wait up to 30 seconds for all processes to indicate completion
+            assert poll_directory(finished_dir, num_workers, 30) is True
+
+            log.warning(f"Finished: {num_workers} workers finished")
 
             # Expect: lock acquisition times out (raises as FileKeyringLockTimeout)
             with pytest.raises(FileKeyringLockTimeout):
-                res.get(timeout=2)
+                res.get(timeout=30)
 
         lock.release_read_lock()
 
     # When: using a new empty keyring
     @using_temp_file_keyring()
-    def test_writer_lock_initially_blocked_by_readers(self):
+    def test_writer_lock_initially_blocked_by_readers(self, ready_dir: Path, finished_dir: Path):
         """
         When a reader lock is already held, another thread/process should not be able
         to acquire the lock for writing until the reader releases its lock
@@ -346,19 +492,38 @@ class TestFileKeyringSynchronization:
 
         child_proc_function = dummy_fn_requiring_writer_lock
         timeout = 1
-        attempts = 4
+        attempts = 10
+        num_workers = 1
 
-        with Pool(processes=1) as pool:
+        with Pool(processes=num_workers) as pool:
             # When: a child process attempts to acquire the same lock for writing, failing after 4 seconds
-            res = pool.starmap_async(child_writer_dispatch, [(child_proc_function, lock_path, timeout, attempts)])
+            res = pool.starmap_async(
+                child_writer_dispatch_with_readiness_check,
+                [(child_proc_function, lock_path, timeout, attempts, ready_dir, finished_dir)],
+            )
+
+            # Wait up to 30 seconds for all processes to indicate readiness
+            assert poll_directory(ready_dir, num_workers, 30) is True
+
+            log.warning(f"Test setup complete: {num_workers} workers ready")
+
+            # Signal that testing should begin
+            start_file_path: Path = ready_dir / "start"
+            with open(start_file_path, "w") as f:
+                f.write(f"{os.getpid()}\n")
 
             # When: we verify that the writer lock is not immediately acquired
             with pytest.raises(TimeoutError):
-                res.get(timeout=1)
+                res.get(timeout=5)
 
             # When: the reader releases its lock
             lock.release_read_lock()
 
+            # Wait up to 30 seconds for all processes to indicate completion
+            assert poll_directory(finished_dir, num_workers, 30) is True
+
+            log.warning(f"Finished: {num_workers} workers finished")
+
             # Expect: the child process to acquire the writer lock
             result = res.get(timeout=10)  # 10 second timeout to prevent a bad test from spoiling the fun
             assert result[0] == "A winner is you!"
