@@ -17,6 +17,8 @@ from chia.util.streamable import (
     Streamable,
     streamable,
     parse_bool,
+    parse_uint32,
+    write_uint32,
     parse_optional,
     parse_bytes,
     parse_list,
@@ -40,7 +42,7 @@ class TestStreamable(unittest.TestCase):
             f: Optional[uint32]
             g: Tuple[uint32, str, bytes]
 
-        a = TestClass(24, 352, [1, 2, 4], [[1, 2, 3], [3, 4]], 728, None, (383, "hello", b"goodbye"))  # type: ignore
+        a = TestClass(24, 352, [1, 2, 4], [[1, 2, 3], [3, 4]], 728, None, (383, "hello", b"goodbye"))
 
         b: bytes = bytes(a)
         assert a == TestClass.from_bytes(b)
@@ -56,16 +58,13 @@ class TestStreamable(unittest.TestCase):
         a = TestClass2(uint32(1), uint32(2), b"3")
         bytes(a)
 
-        try:
+        with raises(NotImplementedError):
 
             @dataclass(frozen=True)
             @streamable
             class TestClass3(Streamable):
                 a: int
 
-        except NotImplementedError:
-            pass
-
     def test_json(self):
         block = bt.create_genesis_block(test_constants, bytes([0] * 32), b"0")
 
@@ -218,6 +217,34 @@ class TestStreamable(unittest.TestCase):
         with raises(ValueError):
             parse_bool(io.BytesIO(b"\x02"))
 
+    def test_uint32(self):
+        assert parse_uint32(io.BytesIO(b"\x00\x00\x00\x00")) == 0
+        assert parse_uint32(io.BytesIO(b"\x00\x00\x00\x01")) == 1
+        assert parse_uint32(io.BytesIO(b"\x00\x00\x00\x01"), "little") == 16777216
+        assert parse_uint32(io.BytesIO(b"\x01\x00\x00\x00")) == 16777216
+        assert parse_uint32(io.BytesIO(b"\x01\x00\x00\x00"), "little") == 1
+        assert parse_uint32(io.BytesIO(b"\xff\xff\xff\xff"), "little") == 4294967295
+
+        def test_write(value, byteorder):
+            f = io.BytesIO()
+            write_uint32(f, uint32(value), byteorder)
+            f.seek(0)
+            assert parse_uint32(f, byteorder) == value
+
+        test_write(1, "big")
+        test_write(1, "little")
+        test_write(4294967295, "big")
+        test_write(4294967295, "little")
+
+        with raises(AssertionError):
+            parse_uint32(io.BytesIO(b""))
+        with raises(AssertionError):
+            parse_uint32(io.BytesIO(b"\x00"))
+        with raises(AssertionError):
+            parse_uint32(io.BytesIO(b"\x00\x00"))
+        with raises(AssertionError):
+            parse_uint32(io.BytesIO(b"\x00\x00\x00"))
+
     def test_parse_optional(self):
         assert parse_optional(io.BytesIO(b"\x00"), parse_bool) is None
         assert parse_optional(io.BytesIO(b"\x01\x01"), parse_bool)
