@@ -1,6 +1,7 @@
 import asyncio
 import atexit
 import signal
+import sqlite3
 
 from secrets import token_bytes
 from typing import Dict, List, Optional
@@ -21,6 +22,7 @@ from chia.types.peer_info import PeerInfo
 from chia.util.bech32m import encode_puzzle_hash
 from tests.block_tools import create_block_tools, create_block_tools_async, test_constants
 from tests.util.keyring import TempKeyring
+from tests.util.socket import find_available_listen_port
 from chia.util.hash import std_hash
 from chia.util.ints import uint16, uint32
 from chia.util.keychain import bytes_to_mnemonic
@@ -36,7 +38,9 @@ keychain = temp_keyring.get_keychain()
 atexit.register(cleanup_keyring, temp_keyring)  # Attempt to cleanup the temp keychain
 bt = create_block_tools(constants=test_constants, keychain=keychain)
 
-self_hostname = bt.config["self_hostname"]
+# if you have a system that has an unusual hostname for localhost and you want
+# to run the tests, change this constant
+self_hostname = "localhost"
 
 
 def constants_for_dic(dic):
@@ -74,16 +78,25 @@ async def setup_full_node(
     consensus_constants: ConsensusConstants,
     db_name,
     port,
+    rpc_port,
     local_bt,
     introducer_port=None,
     simulator=False,
     send_uncompact_interval=0,
     sanitize_weight_proof_only=False,
     connect_to_daemon=False,
+    db_version=1,
 ):
     db_path = local_bt.root_path / f"{db_name}"
     if db_path.exists():
         db_path.unlink()
+
+        if db_version > 1:
+            with sqlite3.connect(db_path) as connection:
+                connection.execute("CREATE TABLE database_version(version int)")
+                connection.execute("INSERT INTO database_version VALUES (?)", (db_version,))
+                connection.commit()
+
     config = local_bt.config["full_node"]
     config["database_path"] = db_name
     config["send_uncompact_interval"] = send_uncompact_interval
@@ -97,7 +110,7 @@ async def setup_full_node(
         config["introducer_peer"] = None
     config["dns_servers"] = []
     config["port"] = port
-    config["rpc_port"] = port + 1000
+    config["rpc_port"] = rpc_port
     overrides = config["network_overrides"]["constants"][config["selected_network"]]
     updated_constants = consensus_constants.replace_str_to_bytes(**overrides)
     if simulator:
@@ -108,9 +121,10 @@ async def setup_full_node(
     kwargs.update(
         parse_cli_args=False,
         connect_to_daemon=connect_to_daemon,
+        service_name_prefix="test_",
     )
 
-    service = Service(**kwargs)
+    service = Service(**kwargs, handle_signals=False)
 
     await service.start()
 
@@ -124,20 +138,22 @@ async def setup_full_node(
 
 async def setup_wallet_node(
     port,
+    rpc_port,
     consensus_constants: ConsensusConstants,
     local_bt,
     full_node_port=None,
     introducer_port=None,
     key_seed=None,
     starting_height=None,
+    initial_num_public_keys=5,
 ):
     with TempKeyring() as keychain:
         config = bt.config["wallet"]
         config["port"] = port
-        config["rpc_port"] = port + 1000
+        config["rpc_port"] = rpc_port
         if starting_height is not None:
             config["starting_height"] = starting_height
-        config["initial_num_public_keys"] = 5
+        config["initial_num_public_keys"] = initial_num_public_keys
 
         entropy = token_bytes(32)
         if key_seed is None:
@@ -173,11 +189,12 @@ async def setup_wallet_node(
         kwargs.update(
             parse_cli_args=False,
             connect_to_daemon=False,
+            service_name_prefix="test_",
         )
 
-        service = Service(**kwargs)
+        service = Service(**kwargs, handle_signals=False)
 
-        await service.start(new_wallet=True)
+        await service.start()
 
         yield service._node, service._node.server
 
@@ -188,21 +205,29 @@ async def setup_wallet_node(
         keychain.delete_all_keys()
 
 
-async def setup_harvester(port, farmer_port, consensus_constants: ConsensusConstants, b_tools):
-    kwargs = service_kwargs_for_harvester(b_tools.root_path, b_tools.config["harvester"], consensus_constants)
+async def setup_harvester(
+    port, rpc_port, farmer_port, consensus_constants: ConsensusConstants, b_tools, start_service: bool = True
+):
+
+    config = bt.config["harvester"]
+    config["port"] = port
+    config["rpc_port"] = rpc_port
+    kwargs = service_kwargs_for_harvester(b_tools.root_path, config, consensus_constants)
     kwargs.update(
         server_listen_ports=[port],
         advertised_port=port,
         connect_peers=[PeerInfo(self_hostname, farmer_port)],
         parse_cli_args=False,
         connect_to_daemon=False,
+        service_name_prefix="test_",
     )
 
-    service = Service(**kwargs)
+    service = Service(**kwargs, handle_signals=False)
 
-    await service.start()
+    if start_service:
+        await service.start()
 
-    yield service._node, service._node.server
+    yield service
 
     service.stop()
     await service.wait_closed()
@@ -210,9 +235,11 @@ async def setup_harvester(port, farmer_port, consensus_constants: ConsensusConst
 
 async def setup_farmer(
     port,
+    rpc_port,
     consensus_constants: ConsensusConstants,
     b_tools,
     full_node_port: Optional[uint16] = None,
+    start_service: bool = True,
 ):
     config = bt.config["farmer"]
     config_pool = bt.config["pool"]
@@ -220,6 +247,7 @@ async def setup_farmer(
     config["xch_target_address"] = encode_puzzle_hash(b_tools.farmer_ph, "xch")
     config["pool_public_keys"] = [bytes(pk).hex() for pk in b_tools.pool_pubkeys]
     config["port"] = port
+    config["rpc_port"] = rpc_port
     config_pool["xch_target_address"] = encode_puzzle_hash(b_tools.pool_ph, "xch")
 
     if full_node_port:
@@ -234,13 +262,15 @@ async def setup_farmer(
     kwargs.update(
         parse_cli_args=False,
         connect_to_daemon=False,
+        service_name_prefix="test_",
     )
 
-    service = Service(**kwargs)
+    service = Service(**kwargs, handle_signals=False)
 
-    await service.start()
+    if start_service:
+        await service.start()
 
-    yield service._api, service._node.server
+    yield service
 
     service.stop()
     await service.wait_closed()
@@ -255,9 +285,10 @@ async def setup_introducer(port):
         advertised_port=port,
         parse_cli_args=False,
         connect_to_daemon=False,
+        service_name_prefix="test_",
     )
 
-    service = Service(**kwargs)
+    service = Service(**kwargs, handle_signals=False)
 
     await service.start()
 
@@ -268,7 +299,7 @@ async def setup_introducer(port):
 
 
 async def setup_vdf_client(port):
-    vdf_task_1 = asyncio.create_task(spawn_process(self_hostname, port, 1))
+    vdf_task_1 = asyncio.create_task(spawn_process(self_hostname, port, 1, bt.config.get("prefer_ipv6")))
 
     def stop():
         asyncio.create_task(kill_processes())
@@ -281,9 +312,9 @@ async def setup_vdf_client(port):
 
 
 async def setup_vdf_clients(port):
-    vdf_task_1 = asyncio.create_task(spawn_process(self_hostname, port, 1))
-    vdf_task_2 = asyncio.create_task(spawn_process(self_hostname, port, 2))
-    vdf_task_3 = asyncio.create_task(spawn_process(self_hostname, port, 3))
+    vdf_task_1 = asyncio.create_task(spawn_process(self_hostname, port, 1, bt.config.get("prefer_ipv6")))
+    vdf_task_2 = asyncio.create_task(spawn_process(self_hostname, port, 2, bt.config.get("prefer_ipv6")))
+    vdf_task_3 = asyncio.create_task(spawn_process(self_hostname, port, 3, bt.config.get("prefer_ipv6")))
 
     def stop():
         asyncio.create_task(kill_processes())
@@ -296,22 +327,26 @@ async def setup_vdf_clients(port):
     await kill_processes()
 
 
-async def setup_timelord(port, full_node_port, sanitizer, consensus_constants: ConsensusConstants, b_tools):
+async def setup_timelord(
+    port, full_node_port, rpc_port, vdf_port, sanitizer, consensus_constants: ConsensusConstants, b_tools
+):
     config = b_tools.config["timelord"]
     config["port"] = port
     config["full_node_peer"]["port"] = full_node_port
-    config["sanitizer_mode"] = sanitizer
+    config["bluebox_mode"] = sanitizer
     config["fast_algorithm"] = False
-    if sanitizer:
-        config["vdf_server"]["port"] = 7999
+    config["vdf_server"]["port"] = vdf_port
+    config["start_rpc_server"] = True
+    config["rpc_port"] = rpc_port
 
     kwargs = service_kwargs_for_timelord(b_tools.root_path, config, consensus_constants)
     kwargs.update(
         parse_cli_args=False,
         connect_to_daemon=False,
+        service_name_prefix="test_",
     )
 
-    service = Service(**kwargs)
+    service = Service(**kwargs, handle_signals=False)
 
     await service.start()
 
@@ -321,7 +356,7 @@ async def setup_timelord(port, full_node_port, sanitizer, consensus_constants: C
     await service.wait_closed()
 
 
-async def setup_two_nodes(consensus_constants: ConsensusConstants):
+async def setup_two_nodes(consensus_constants: ConsensusConstants, db_version: int):
     """
     Setup and teardown of two full nodes, with blockchains and separate DBs.
     """
@@ -331,16 +366,20 @@ async def setup_two_nodes(consensus_constants: ConsensusConstants):
             setup_full_node(
                 consensus_constants,
                 "blockchain_test.db",
-                21234,
+                find_available_listen_port("node1"),
+                find_available_listen_port("node1 rpc"),
                 await create_block_tools_async(constants=test_constants, keychain=keychain1),
                 simulator=False,
+                db_version=db_version,
             ),
             setup_full_node(
                 consensus_constants,
                 "blockchain_test_2.db",
-                21235,
+                find_available_listen_port("node2"),
+                find_available_listen_port("node2 rpc"),
                 await create_block_tools_async(constants=test_constants, keychain=keychain2),
                 simulator=False,
+                db_version=db_version,
             ),
         ]
 
@@ -352,11 +391,10 @@ async def setup_two_nodes(consensus_constants: ConsensusConstants):
         await _teardown_nodes(node_iters)
 
 
-async def setup_n_nodes(consensus_constants: ConsensusConstants, n: int):
+async def setup_n_nodes(consensus_constants: ConsensusConstants, n: int, db_version: int):
     """
     Setup and teardown of n full nodes, with blockchains and separate DBs.
     """
-    port_start = 21244
     node_iters = []
     keyrings_to_cleanup = []
     for i in range(n):
@@ -366,9 +404,11 @@ async def setup_n_nodes(consensus_constants: ConsensusConstants, n: int):
             setup_full_node(
                 consensus_constants,
                 f"blockchain_test_{i}.db",
-                port_start + i,
+                find_available_listen_port(f"node{i}"),
+                find_available_listen_port(f"node{i} rpc"),
                 await create_block_tools_async(constants=test_constants, keychain=keyring.get_keychain()),
                 simulator=False,
+                db_version=db_version,
             )
         )
     nodes = []
@@ -383,13 +423,29 @@ async def setup_n_nodes(consensus_constants: ConsensusConstants, n: int):
         keyring.cleanup()
 
 
-async def setup_node_and_wallet(consensus_constants: ConsensusConstants, starting_height=None, key_seed=None):
+async def setup_node_and_wallet(
+    consensus_constants: ConsensusConstants, starting_height=None, key_seed=None, db_version=1
+):
     with TempKeyring() as keychain:
         btools = await create_block_tools_async(constants=test_constants, keychain=keychain)
         node_iters = [
-            setup_full_node(consensus_constants, "blockchain_test.db", 21234, btools, simulator=False),
+            setup_full_node(
+                consensus_constants,
+                "blockchain_test.db",
+                find_available_listen_port("node1"),
+                find_available_listen_port("node1 rpc"),
+                btools,
+                simulator=False,
+                db_version=db_version,
+            ),
             setup_wallet_node(
-                21235, consensus_constants, btools, None, starting_height=starting_height, key_seed=key_seed
+                find_available_listen_port("node2"),
+                find_available_listen_port("node2 rpc"),
+                consensus_constants,
+                btools,
+                None,
+                starting_height=starting_height,
+                key_seed=key_seed,
             ),
         ]
 
@@ -407,7 +463,8 @@ async def setup_simulators_and_wallets(
     dic: Dict,
     starting_height=None,
     key_seed=None,
-    starting_port=50000,
+    initial_num_public_keys=5,
+    db_version=1,
 ):
     with TempKeyring() as keychain1, TempKeyring() as keychain2:
         simulators: List[FullNodeAPI] = []
@@ -416,7 +473,8 @@ async def setup_simulators_and_wallets(
 
         consensus_constants = constants_for_dic(dic)
         for index in range(0, simulator_count):
-            port = starting_port + index
+            port = find_available_listen_port(f"node{index}")
+            rpc_port = find_available_listen_port(f"node{index} rpc")
             db_name = f"blockchain_test_{port}.db"
             bt_tools = await create_block_tools_async(
                 consensus_constants, const_dict=dic, keychain=keychain1
@@ -425,8 +483,10 @@ async def setup_simulators_and_wallets(
                 bt_tools.constants,
                 db_name,
                 port,
+                rpc_port,
                 bt_tools,
                 simulator=True,
+                db_version=db_version,
             )
             simulators.append(await sim.__anext__())
             node_iters.append(sim)
@@ -436,17 +496,20 @@ async def setup_simulators_and_wallets(
                 seed = std_hash(uint32(index))
             else:
                 seed = key_seed
-            port = starting_port + 5000 + index
+            port = find_available_listen_port(f"wallet{index}")
+            rpc_port = find_available_listen_port(f"wallet{index} rpc")
             bt_tools = await create_block_tools_async(
                 consensus_constants, const_dict=dic, keychain=keychain2
             )  # block tools modifies constants
             wlt = setup_wallet_node(
                 port,
+                rpc_port,
                 bt_tools.constants,
                 bt_tools,
                 None,
                 key_seed=seed,
                 starting_height=starting_height,
+                initial_num_public_keys=initial_num_public_keys,
             )
             wallets.append(await wlt.__anext__())
             node_iters.append(wlt)
@@ -456,47 +519,94 @@ async def setup_simulators_and_wallets(
         await _teardown_nodes(node_iters)
 
 
-async def setup_farmer_harvester(consensus_constants: ConsensusConstants):
+async def setup_farmer_harvester(consensus_constants: ConsensusConstants, start_services: bool = True):
+    farmer_port = find_available_listen_port("farmer")
+    farmer_rpc_port = find_available_listen_port("farmer rpc")
+    harvester_port = find_available_listen_port("harvester")
+    harvester_rpc_port = find_available_listen_port("harvester rpc")
     node_iters = [
-        setup_harvester(21234, 21235, consensus_constants, bt),
-        setup_farmer(21235, consensus_constants, bt),
+        setup_harvester(harvester_port, harvester_rpc_port, farmer_port, consensus_constants, bt, start_services),
+        setup_farmer(farmer_port, farmer_rpc_port, consensus_constants, bt, start_service=start_services),
     ]
 
-    harvester, harvester_server = await node_iters[0].__anext__()
-    farmer, farmer_server = await node_iters[1].__anext__()
+    harvester_service = await node_iters[0].__anext__()
+    farmer_service = await node_iters[1].__anext__()
 
-    yield harvester, farmer
+    yield harvester_service, farmer_service
 
     await _teardown_nodes(node_iters)
 
 
 async def setup_full_system(
-    consensus_constants: ConsensusConstants, b_tools=None, b_tools_1=None, connect_to_daemon=False
+    consensus_constants: ConsensusConstants, b_tools=None, b_tools_1=None, connect_to_daemon=False, db_version=1
 ):
     with TempKeyring() as keychain1, TempKeyring() as keychain2:
         if b_tools is None:
             b_tools = await create_block_tools_async(constants=test_constants, keychain=keychain1)
         if b_tools_1 is None:
             b_tools_1 = await create_block_tools_async(constants=test_constants, keychain=keychain2)
+        introducer_port = find_available_listen_port("introducer")
+        farmer_port = find_available_listen_port("farmer")
+        farmer_rpc_port = find_available_listen_port("farmer rpc")
+        node1_port = find_available_listen_port("node1")
+        rpc1_port = find_available_listen_port("node1 rpc")
+        node2_port = find_available_listen_port("node2")
+        rpc2_port = find_available_listen_port("node2 rpc")
+        timelord1_port = find_available_listen_port("timelord1")
+        timelord1_rpc_port = find_available_listen_port("timelord1 rpc")
+        timelord2_port = find_available_listen_port("timelord2")
+        timelord2_rpc_port = find_available_listen_port("timelord2 rpc")
+        vdf1_port = find_available_listen_port("vdf1")
+        vdf2_port = find_available_listen_port("vdf2")
+        harvester_port = find_available_listen_port("harvester")
+        harvester_rpc_port = find_available_listen_port("harvester rpc")
+
         node_iters = [
-            setup_introducer(21233),
-            setup_harvester(21234, 21235, consensus_constants, b_tools),
-            setup_farmer(21235, consensus_constants, b_tools, uint16(21237)),
-            setup_vdf_clients(8000),
-            setup_timelord(21236, 21237, False, consensus_constants, b_tools),
-            setup_full_node(
-                consensus_constants, "blockchain_test.db", 21237, b_tools, 21233, False, 10, True, connect_to_daemon
+            setup_introducer(introducer_port),
+            setup_harvester(harvester_port, harvester_rpc_port, farmer_port, consensus_constants, b_tools),
+            setup_farmer(farmer_port, farmer_rpc_port, consensus_constants, b_tools, uint16(node1_port)),
+            setup_vdf_clients(vdf1_port),
+            setup_timelord(
+                timelord2_port, node1_port, timelord2_rpc_port, vdf1_port, False, consensus_constants, b_tools
             ),
             setup_full_node(
-                consensus_constants, "blockchain_test_2.db", 21238, b_tools_1, 21233, False, 10, True, connect_to_daemon
+                consensus_constants,
+                "blockchain_test.db",
+                node1_port,
+                rpc1_port,
+                b_tools,
+                introducer_port,
+                False,
+                10,
+                True,
+                connect_to_daemon,
+                db_version=db_version,
             ),
-            setup_vdf_client(7999),
-            setup_timelord(21239, 21238, True, consensus_constants, b_tools_1),
+            setup_full_node(
+                consensus_constants,
+                "blockchain_test_2.db",
+                node2_port,
+                rpc2_port,
+                b_tools_1,
+                introducer_port,
+                False,
+                10,
+                True,
+                False,  # connect_to_daemon,
+                db_version=db_version,
+            ),
+            setup_vdf_client(vdf2_port),
+            setup_timelord(timelord1_port, 1000, timelord1_rpc_port, vdf2_port, True, consensus_constants, b_tools_1),
         ]
 
+        if connect_to_daemon:
+            node_iters.append(setup_daemon(btools=b_tools))
+
         introducer, introducer_server = await node_iters[0].__anext__()
-        harvester, harvester_server = await node_iters[1].__anext__()
-        farmer, farmer_server = await node_iters[2].__anext__()
+        harvester_service = await node_iters[1].__anext__()
+        harvester = harvester_service._node
+        farmer_service = await node_iters[2].__anext__()
+        farmer = farmer_service._node
 
         async def num_connections():
             count = len(harvester.server.all_connections.items())
@@ -511,7 +621,7 @@ async def setup_full_system(
         vdf_sanitizer = await node_iters[7].__anext__()
         sanitizer, sanitizer_server = await node_iters[8].__anext__()
 
-        yield (
+        ret = (
             node_api_1,
             node_api_2,
             harvester,
@@ -521,7 +631,14 @@ async def setup_full_system(
             vdf_clients,
             vdf_sanitizer,
             sanitizer,
+            sanitizer_server,
             node_api_1.full_node.server,
         )
 
+        if connect_to_daemon:
+            daemon1 = await node_iters[9].__anext__()
+            yield ret + (daemon1,)
+        else:
+            yield ret
+
         await _teardown_nodes(node_iters)
