@@ -1,4 +1,5 @@
 import pytest
+import pytest_asyncio
 
 from chia.types.peer_info import PeerInfo
 from tests.block_tools import create_block_tools_async
@@ -7,6 +8,7 @@ from tests.core.node_height import node_height_at_least
 from tests.setup_nodes import self_hostname, setup_full_node, setup_full_system, test_constants
 from tests.time_out_assert import time_out_assert
 from tests.util.keyring import TempKeyring
+from tests.util.socket import find_available_listen_port
 
 test_constants_modified = test_constants.replace(
     **{
@@ -25,30 +27,46 @@ test_constants_modified = test_constants.replace(
 
 
 class TestSimulation:
-    @pytest.fixture(scope="function")
+
+    # TODO: Ideally, the db_version should be the (parameterized) db_version
+    # fixture, to test all versions of the database schema. This doesn't work
+    # because of a hack in shutting down the full node, which means you cannot run
+    # more than one simulations per process.
+    @pytest_asyncio.fixture(scope="function")
     async def extra_node(self):
         with TempKeyring() as keychain:
             b_tools = await create_block_tools_async(constants=test_constants_modified, keychain=keychain)
-            async for _ in setup_full_node(test_constants_modified, "blockchain_test_3.db", 21240, b_tools):
+            async for _ in setup_full_node(
+                test_constants_modified,
+                "blockchain_test_3.db",
+                find_available_listen_port(),
+                find_available_listen_port(),
+                b_tools,
+                db_version=1,
+            ):
                 yield _
 
-    @pytest.fixture(scope="function")
+    @pytest_asyncio.fixture(scope="function")
     async def simulation(self):
-        async for _ in setup_full_system(test_constants_modified):
+        async for _ in setup_full_system(test_constants_modified, db_version=1):
             yield _
 
     @pytest.mark.asyncio
     async def test_simulation_1(self, simulation, extra_node):
-        node1, node2, _, _, _, _, _, _, _, server1 = simulation
-        await server1.start_client(PeerInfo(self_hostname, uint16(21238)))
+        node1, node2, _, _, _, _, _, _, _, sanitizer_server, server1 = simulation
+
+        node1_port = node1.full_node.config["port"]
+        node2_port = node2.full_node.config["port"]
+        await server1.start_client(PeerInfo(self_hostname, uint16(node2_port)))
         # Use node2 to test node communication, since only node1 extends the chain.
-        await time_out_assert(1500, node_height_at_least, True, node2, 7)
+        await time_out_assert(600, node_height_at_least, True, node2, 7)
+        await sanitizer_server.start_client(PeerInfo(self_hostname, uint16(node2_port)))
 
         async def has_compact(node1, node2):
             peak_height_1 = node1.full_node.blockchain.get_peak_height()
-            headers_1 = await node1.full_node.blockchain.get_header_blocks_in_range(0, peak_height_1)
+            headers_1 = await node1.full_node.blockchain.get_header_blocks_in_range(0, peak_height_1 - 6)
             peak_height_2 = node2.full_node.blockchain.get_peak_height()
-            headers_2 = await node2.full_node.blockchain.get_header_blocks_in_range(0, peak_height_2)
+            headers_2 = await node2.full_node.blockchain.get_header_blocks_in_range(0, peak_height_2 - 6)
             # Commented to speed up.
             # cc_eos = [False, False]
             # icc_eos = [False, False]
@@ -82,10 +100,10 @@ class TestSimulation:
             # )
             return has_compact == [True, True]
 
-        await time_out_assert(1500, has_compact, True, node1, node2)
+        await time_out_assert(600, has_compact, True, node1, node2)
         node3 = extra_node
         server3 = node3.full_node.server
         peak_height = max(node1.full_node.blockchain.get_peak_height(), node2.full_node.blockchain.get_peak_height())
-        await server3.start_client(PeerInfo(self_hostname, uint16(21237)))
-        await server3.start_client(PeerInfo(self_hostname, uint16(21238)))
+        await server3.start_client(PeerInfo(self_hostname, uint16(node1_port)))
+        await server3.start_client(PeerInfo(self_hostname, uint16(node2_port)))
         await time_out_assert(600, node_height_at_least, True, node3, peak_height)
