@@ -3,17 +3,17 @@ import shutil
 import tempfile
 
 from chia.util.file_keyring import FileKeyring
-from chia.util.keychain import Keychain
+from chia.util.keychain import Keychain, default_keychain_service, default_keychain_user, get_private_key_user
 from chia.util.keyring_wrapper import KeyringWrapper
 from functools import wraps
 from keyring.util import platform_
 from keyrings.cryptfile.cryptfile import CryptFileKeyring  # pyright: reportMissingImports=false
 from pathlib import Path
-from typing import Optional
+from typing import Any, Optional
 from unittest.mock import patch
 
 
-def create_empty_cryptfilekeyring():
+def create_empty_cryptfilekeyring() -> CryptFileKeyring:
     """
     Create an empty legacy keyring
     """
@@ -21,6 +21,16 @@ def create_empty_cryptfilekeyring():
     fd = os.open(crypt_file_keyring.file_path, os.O_CREAT | os.O_WRONLY | os.O_TRUNC, 0o600)
     os.close(fd)
     assert Path(crypt_file_keyring.file_path).exists()
+    return crypt_file_keyring
+
+
+def add_dummy_key_to_cryptfilekeyring(crypt_file_keyring: CryptFileKeyring):
+    """
+    Add a fake key to the CryptFileKeyring
+    """
+    crypt_file_keyring.keyring_key = "your keyring password"
+    user: str = get_private_key_user(default_keychain_user(), 0)
+    crypt_file_keyring.set_password(default_keychain_service(), user, "abc123")
 
 
 def setup_mock_file_keyring(mock_configure_backend, temp_file_keyring_dir, populate=False):
@@ -77,9 +87,7 @@ def using_temp_file_keyring_and_cryptfilekeyring(populate=False):
     def outer(method):
         @wraps(method)
         def inner(self, *args, **kwargs):
-            with TempKeyring(populate=populate):
-                # Create an empty legacy keyring
-                create_empty_cryptfilekeyring()
+            with TempKeyring(populate=populate, setup_cryptfilekeyring=True):
                 return method(self, *args, **kwargs)
 
         return inner
@@ -90,24 +98,33 @@ def using_temp_file_keyring_and_cryptfilekeyring(populate=False):
 class TempKeyring:
     def __init__(
         self,
+        *,
         user: str = "testing-1.8.0",
         service: str = "testing-chia-1.8.0",
         populate: bool = False,
+        setup_cryptfilekeyring: bool = False,
         existing_keyring_path: str = None,
         delete_on_cleanup: bool = True,
         use_os_credential_store: bool = False,
     ):
         self.keychain = self._patch_and_create_keychain(
-            user, service, populate, existing_keyring_path, use_os_credential_store
+            user=user,
+            service=service,
+            populate=populate,
+            existing_keyring_path=existing_keyring_path,
+            use_os_credential_store=use_os_credential_store,
+            setup_cryptfilekeyring=setup_cryptfilekeyring,
         )
         self.delete_on_cleanup = delete_on_cleanup
         self.cleaned_up = False
 
     def _patch_and_create_keychain(
         self,
+        *,
         user: str,
         service: str,
         populate: bool,
+        setup_cryptfilekeyring: bool,
         existing_keyring_path: Optional[str],
         use_os_credential_store: bool,
     ):
@@ -130,9 +147,11 @@ class TempKeyring:
         mock_configure_backend = mock_configure_backend_patch.start()
         setup_mock_file_keyring(mock_configure_backend, temp_dir, populate=populate)
 
-        mock_configure_legacy_backend_patch = patch.object(KeyringWrapper, "_configure_legacy_backend")
-        mock_configure_legacy_backend = mock_configure_legacy_backend_patch.start()
-        mock_configure_legacy_backend.return_value = None
+        mock_configure_legacy_backend_patch: Any = None
+        if setup_cryptfilekeyring is False:
+            mock_configure_legacy_backend_patch = patch.object(KeyringWrapper, "_configure_legacy_backend")
+            mock_configure_legacy_backend = mock_configure_legacy_backend_patch.start()
+            mock_configure_legacy_backend.return_value = None
 
         mock_data_root_patch = patch.object(platform_, "data_root")
         mock_data_root = mock_data_root_patch.start()
@@ -141,6 +160,10 @@ class TempKeyring:
         # We don't want CryptFileKeyring finding the real legacy keyring
         mock_data_root.return_value = temp_dir
 
+        if setup_cryptfilekeyring is True:
+            crypt_file_keyring = create_empty_cryptfilekeyring()
+            add_dummy_key_to_cryptfilekeyring(crypt_file_keyring)
+
         keychain = Keychain(user=user, service=service)
         keychain.keyring_wrapper = KeyringWrapper(keys_root_path=Path(temp_dir))
 
@@ -171,14 +194,13 @@ class TempKeyring:
 
         if self.delete_on_cleanup:
             self.keychain.keyring_wrapper.keyring.cleanup_keyring_file_watcher()
-            temp_dir = self.keychain._temp_dir
-            print(f"Cleaning up temp keychain in dir: {temp_dir}")
-            shutil.rmtree(temp_dir)
+            shutil.rmtree(self.keychain._temp_dir)
 
         self.keychain._mock_supports_keyring_passphrase_patch.stop()
         self.keychain._mock_supports_os_passphrase_storage_patch.stop()
         self.keychain._mock_configure_backend_patch.stop()
-        self.keychain._mock_configure_legacy_backend_patch.stop()
+        if self.keychain._mock_configure_legacy_backend_patch is not None:
+            self.keychain._mock_configure_legacy_backend_patch.stop()
         self.keychain._mock_data_root_patch.stop()
 
         self.cleaned_up = True
